@@ -1,14 +1,14 @@
 /* $RoughId: md5init.c,v 1.2 2001/07/13 19:49:10 knu Exp $ */
-/* $Id: md5init.c 11708 2007-02-12 23:01:19Z shyouhei $ */
+/* $Id: md5init.c 28341 2010-06-16 09:38:14Z knu $ */
 
-#include "digest.h"
+#include "../digest.h"
 #if defined(HAVE_OPENSSL_MD5_H)
 #include "md5ossl.h"
 #else
 #include "md5.h"
 #endif
 
-static rb_digest_metadata_t md5 = {
+static const rb_digest_metadata_t md5 = {
     RUBY_DIGEST_API_VERSION,
     MD5_DIGEST_LENGTH,
     MD5_BLOCK_LENGTH,
@@ -36,5 +36,5 @@ Init_md5()
     cDigest_MD5 = rb_define_class_under(mDigest, "MD5", cDigest_Base);
 
     rb_ivar_set(cDigest_MD5, rb_intern("metadata"),
-      Data_Wrap_Struct(rb_cObject, 0, 0, &md5));
+      Data_Wrap_Struct(rb_cObject, 0, 0, (void *)&md5));
 }
