@@ -2,13 +2,13 @@
 
   readline.c - GNU Readline module
 
-  $Author: kouji $
+  $Author: yugui $
   created at: Wed Jan 20 13:59:32 JST 1999
 
   Copyright (C) 1997-2008  Shugo Maeda
-  Copyright (C) 2008       TAKAO Kouji
+  Copyright (C) 2008-2009  TAKAO Kouji
 
-  $Id: readline.c 19456 2008-09-22 17:07:22Z kouji $
+  $Id: readline.c 30569 2011-01-16 12:35:04Z yugui $
 
   Contact:
    - TAKAO Kouji <kouji at takao7 dot net> (current maintainer)
@@ -43,10 +43,20 @@
 static VALUE mReadline;
 
 #define EDIT_LINE_LIBRARY_VERSION "EditLine wrapper"
+#ifndef USE_INSERT_IGNORE_ESCAPE
+# if !defined(HAVE_EDITLINE_READLINE_H) && defined(HAVE_RL_PROMPT_START_IGNORE) && defined(HAVE_RL_PROMPT_END_IGNORE)
+#  define USE_INSERT_IGNORE_ESCAPE 1
+# else
+#  define USE_INSERT_IGNORE_ESCAPE 0
+# endif
+#endif
 
 #define COMPLETION_PROC "completion_proc"
 #define COMPLETION_CASE_FOLD "completion_case_fold"
 static ID completion_proc, completion_case_fold;
+#if USE_INSERT_IGNORE_ESCAPE
+static ID id_orig_prompt, id_last_prompt;
+#endif
 
 #ifndef HAVE_RL_FILENAME_COMPLETION_FUNCTION
 # define rl_filename_completion_function filename_completion_function
@@ -63,8 +73,11 @@ static int (*history_get_offset_func)(int);
 static char **readline_attempted_completion_function(const char *text,
                                                      int start, int end);
 
-#ifdef HAVE_RL_EVENT_HOOK
-#define BUSY_WAIT 0
+#define OutputStringValue(str) do {\
+    SafeStringValue(str);\
+    str = rb_str_conv_enc(str, rb_enc_get(str), rb_locale_encoding());\
+} while (0)\
+
 
 /*
  * Document-class: Readline
@@ -78,7 +91,7 @@ static char **readline_attempted_completion_function(const char *text,
  * GNU Readline:: http://www.gnu.org/directory/readline.html
  * libedit::      http://www.thrysoee.dk/editline/
  *
- * Reads one inputted line with line edit by Readline.readline method. 
+ * Reads one inputted line with line edit by Readline.readline method.
  * At this time, the facilitatation completion and the key
  * bind like Emacs can be operated like GNU Readline.
  *
@@ -101,6 +114,26 @@ static char **readline_attempted_completion_function(const char *text,
  * Documented by TAKAO Kouji <kouji at takao7 dot net>.
  */
 
+#if defined HAVE_RL_GETC_FUNCTION
+static VALUE readline_instream;
+static ID id_getbyte;
+
+static int readline_getc(FILE *);
+static int
+readline_getc(FILE *input)
+{
+    rb_io_t *ifp = 0;
+    VALUE c;
+    if (!readline_instream) return rl_getc(input);
+    GetOpenFile(readline_instream, ifp);
+    if (rl_instream != ifp->stdio_file) return rl_getc(input);
+    c = rb_funcall(readline_instream, id_getbyte, 0, 0);
+    if (NIL_P(c)) return EOF;
+    return NUM2CHR(c);
+}
+#elif defined HAVE_RL_EVENT_HOOK
+#define BUSY_WAIT 0
+
 static int readline_event(void);
 static int
 readline_event(void)
@@ -118,6 +151,87 @@ readline_event(void)
 }
 #endif
 
+#if USE_INSERT_IGNORE_ESCAPE
+static VALUE
+insert_ignore_escape(VALUE self, VALUE prompt)
+{
+    VALUE last_prompt, orig_prompt = rb_attr_get(self, id_orig_prompt);
+    int ignoring = 0;
+    const char *s0, *s, *e;
+    long len;
+    static const char ignore_code[2] = {RL_PROMPT_START_IGNORE, RL_PROMPT_END_IGNORE};
+
+    prompt = rb_str_new_shared(prompt);
+    last_prompt = rb_attr_get(self, id_last_prompt);
+    if (orig_prompt == prompt) return last_prompt;
+    len = RSTRING_LEN(prompt);
+    if (NIL_P(last_prompt)) {
+	last_prompt = rb_str_tmp_new(len);
+    }
+
+    s = s0 = RSTRING_PTR(prompt);
+    e = s0 + len;
+    rb_str_set_len(last_prompt, 0);
+    while (s < e && *s) {
+	switch (*s) {
+	  case RL_PROMPT_START_IGNORE:
+	    ignoring = -1;
+	    rb_str_cat(last_prompt, s0, ++s - s0);
+	    s0 = s;
+	    break;
+	  case RL_PROMPT_END_IGNORE:
+	    ignoring = 0;
+	    rb_str_cat(last_prompt, s0, ++s - s0);
+	    s0 = s;
+	    break;
+	  case '\033':
+	    if (++s < e && *s == '[') {
+		rb_str_cat(last_prompt, s0, s - s0 - 1);
+		s0 = s - 1;
+		while (++s < e && *s) {
+		    if (ISALPHA(*s)) {
+			if (!ignoring) {
+			    ignoring = 1;
+			    rb_str_cat(last_prompt, ignore_code+0, 1);
+			}
+			rb_str_cat(last_prompt, s0, ++s - s0);
+			s0 = s;
+			break;
+		    }
+		    else if (!('0' <= *s && *s <= '9' || *s == ';')) {
+			break;
+		    }
+		}
+	    }
+	    break;
+	  default:
+	    if (ignoring > 0) {
+		ignoring = 0;
+		rb_str_cat(last_prompt, ignore_code+1, 1);
+	    }
+	    s++;
+	    break;
+	}
+    }
+    if (ignoring > 0) {
+	ignoring = 0;
+	rb_str_cat(last_prompt, ignore_code+1, 1);
+    }
+    rb_str_cat(last_prompt, s0, s - s0);
+
+    rb_ivar_set(self, id_orig_prompt, prompt);
+    rb_ivar_set(self, id_last_prompt, last_prompt);
+
+    return last_prompt;
+}
+#endif
+
+static VALUE
+readline_get(VALUE prompt)
+{
+    return (VALUE)readline((char *)prompt);
+}
+
 /*
  * call-seq:
  *   Readline.readline(prompt = "", add_hist = false) -> string or nil
@@ -144,9 +258,9 @@ readline_event(void)
  *
  * * Catches the Interrupt exception by pressed ^C after returns
  *   terminal status:
- * 
+ *
  *     require "readline"
- *     
+ *
  *     stty_save = `stty -g`.chomp
  *     begin
  *       while buf = Readline.readline
@@ -158,25 +272,25 @@ readline_event(void)
  *         end
  *       end
  *     end
- * 
+ *
  * * Catches the INT signal by pressed ^C after returns terminal
  *   status:
- * 
+ *
  *     require "readline"
- *     
+ *
  *     stty_save = `stty -g`.chomp
  *     trap("INT") { system "stty", stty_save; exit }
- *   
+ *
  *     while buf = Readline.readline
  *       p buf
  *     end
  *
  * * Ignores pressing ^C:
- * 
+ *
  *     require "readline"
- *     
+ *
  *     trap("INT", "SIG_IGN")
- *     
+ *
  *     while buf = Readline.readline
  *       p buf
  *     end
@@ -186,18 +300,18 @@ readline_event(void)
  * the same it as last one.
  *
  *   require "readline"
- *     
+ *
  *   while buf = Readline.readline("> ", true)
  *     # p Readline::HISTORY.to_a
  *     Readline::HISTORY.pop if /^\s*$/ =~ buf
- *  
+ *
  *     begin
  *       if Readline::HISTORY[Readline::HISTORY.length-2] == buf
- *         Readline::HISTORY.pop 
+ *         Readline::HISTORY.pop
  *       end
  *     rescue IndexError
  *     end
- *  
+ *
  *     # p Readline::HISTORY.to_a
  *     print "-> ", buf, "\n"
  *   end
@@ -214,17 +328,29 @@ readline_readline(int argc, VALUE *argv, VALUE self)
 
     rb_secure(4);
     if (rb_scan_args(argc, argv, "02", &tmp, &add_hist) > 0) {
-	SafeStringValue(tmp);
+	OutputStringValue(tmp);
+#if USE_INSERT_IGNORE_ESCAPE
+	tmp = insert_ignore_escape(self, tmp);
+	rb_str_locktmp(tmp);
+#endif
 	prompt = RSTRING_PTR(tmp);
     }
 
-    if (!isatty(0) && errno == EBADF) rb_raise(rb_eIOError, "stdin closed");
+    if (!isatty(fileno(rl_instream)) && errno == EBADF) rb_raise(rb_eIOError, "closed stdin");
 
-    buff = (char*)rb_protect((VALUE(*)_((VALUE)))readline, (VALUE)prompt,
-                              &status);
+#ifdef _WIN32
+    rl_prep_terminal(1);
+#endif
+    buff = (char*)rb_protect(readline_get, (VALUE)prompt, &status);
+#if USE_INSERT_IGNORE_ESCAPE
+    if (prompt) {
+	rb_str_unlocktmp(tmp);
+    }
+#endif
     if (status) {
 #if defined HAVE_RL_CLEANUP_AFTER_SIGNAL
         /* restore terminal mode and signal handler*/
+        rl_free_line_state();
         rl_cleanup_after_signal();
 #elif defined HAVE_RL_DEPREP_TERM_FUNCTION
         /* restore terminal mode */
@@ -241,8 +367,7 @@ readline_readline(int argc, VALUE *argv, VALUE self)
 	add_history(buff);
     }
     if (buff) {
-	result = rb_tainted_str_new2(buff);
-	rb_enc_associate(result, rb_locale_encoding());
+	result = rb_locale_str_new_cstr(buff);
     }
     else
 	result = Qnil;
@@ -268,6 +393,9 @@ readline_s_set_input(VALUE self, VALUE input)
     Check_Type(input, T_FILE);
     GetOpenFile(input, ifp);
     rl_instream = rb_io_stdio_file(ifp);
+#ifdef HAVE_RL_GETC_FUNCTION
+    readline_instream = input;
+#endif
     return input;
 }
 
@@ -300,6 +428,8 @@ readline_s_set_output(VALUE self, VALUE output)
  * should take input-string, and return an array of completion
  * candidates.
  *
+ * Set default if +proc+ is nil.
+ *
  * Raises ArgumentError exception if +proc+ does not respond to call method.
  *
  * Raises SecurityError exception if $SAFE is 4.
@@ -308,7 +438,7 @@ static VALUE
 readline_s_set_completion_proc(VALUE self, VALUE proc)
 {
     rb_secure(4);
-    if (!rb_respond_to(proc, rb_intern("call")))
+    if (!NIL_P(proc) && !rb_respond_to(proc, rb_intern("call")))
 	rb_raise(rb_eArgError, "argument must respond to `call'");
     return rb_ivar_set(mReadline, completion_proc, proc);
 }
@@ -353,7 +483,7 @@ readline_s_set_completion_case_fold(VALUE self, VALUE val)
  * Readline.completion_case_fold= method.
  *
  *   require "readline"
- *   
+ *
  *   Readline.completion_case_fold = "This is a String."
  *   p Readline.completion_case_fold # => "This is a String."
  *
@@ -366,13 +496,61 @@ readline_s_get_completion_case_fold(VALUE self)
     return rb_attr_get(mReadline, completion_case_fold);
 }
 
+#ifdef HAVE_RL_LINE_BUFFER
+/*
+ * call-seq:
+ *   Readline.line_buffer -> string
+ *
+ * Returns the full line that is being edited. This is useful from
+ * within the complete_proc for determining the context of the
+ * completion request.
+ *
+ * The length of +Readline.line_buffer+ and GNU Readline's rl_end are
+ * same.
+ */
+static VALUE
+readline_s_get_line_buffer(VALUE self)
+{
+    rb_secure(4);
+    if (rl_line_buffer == NULL)
+	return Qnil;
+    return rb_locale_str_new_cstr(rl_line_buffer);
+}
+#else
+#define readline_s_get_line_buffer rb_f_notimplement
+#endif
+
+#ifdef HAVE_RL_POINT
+/*
+ * call-seq:
+ *   Readline.point -> int
+ *
+ * Returns the index of the current cursor position in
+ * +Readline.line_buffer+.
+ *
+ * The index in +Readline.line_buffer+ which matches the start of
+ * input-string passed to completion_proc is computed by subtracting
+ * the length of input-string from +Readline.point+.
+ *
+ *   start = (the length of input-string) - Readline.point
+ */
+static VALUE
+readline_s_get_point(VALUE self)
+{
+    rb_secure(4);
+    return INT2NUM(rl_point);
+}
+#else
+#define readline_s_get_point rb_f_notimplement
+#endif
+
 static char **
 readline_attempted_completion_function(const char *text, int start, int end)
 {
     VALUE proc, ary, temp;
     char **result;
     int case_fold;
-    int i, matches;
+    long i, matches;
 
     proc = rb_attr_get(mReadline, completion_proc);
     if (NIL_P(proc))
@@ -381,7 +559,7 @@ readline_attempted_completion_function(const char *text, int start, int end)
     rl_attempted_completion_over = 1;
 #endif
     case_fold = RTEST(rb_attr_get(mReadline, completion_case_fold));
-    ary = rb_funcall(proc, rb_intern("call"), 1, rb_tainted_str_new2(text));
+    ary = rb_funcall(proc, rb_intern("call"), 1, rb_locale_str_new_cstr(text));
     if (TYPE(ary) != T_ARRAY)
 	ary = rb_Array(ary);
     matches = RARRAY_LEN(ary);
@@ -430,6 +608,61 @@ readline_attempted_completion_function(const char *text, int start, int end)
     return result;
 }
 
+#ifdef HAVE_RL_SET_SCREEN_SIZE
+/*
+ * call-seq:
+ *   Readline.set_screen_size(rows, columns) -> self
+ *
+ * Set terminal size to +rows+ and +columns+.
+ *
+ * See GNU Readline's rl_set_screen_size function.
+ *
+ * Raises NotImplementedError if the using readline library does not support.
+ *
+ * Raises SecurityError exception if $SAFE is 4.
+ */
+static VALUE
+readline_s_set_screen_size(VALUE self, VALUE rows, VALUE columns)
+{
+    rb_secure(4);
+    rl_set_screen_size(NUM2INT(rows), NUM2INT(columns));
+    return self;
+}
+#else
+#define readline_s_set_screen_size rb_f_notimplement
+#endif
+
+#ifdef HAVE_RL_GET_SCREEN_SIZE
+/*
+ * call-seq:
+ *   Readline.get_screen_size -> [rows, columns]
+ *
+ * Returns the terminal's rows and columns.
+ *
+ * See GNU Readline's rl_get_screen_size function.
+ *
+ * Raises NotImplementedError if the using readline library does not support.
+ *
+ * Raises SecurityError exception if $SAFE is 4.
+ */
+static VALUE
+readline_s_get_screen_size(VALUE self)
+{
+    int rows, columns;
+    VALUE res;
+
+    rb_secure(4);
+    rl_get_screen_size(&rows, &columns);
+    res = rb_ary_new();
+    rb_ary_push(res, INT2NUM(rows));
+    rb_ary_push(res, INT2NUM(columns));
+    return res;
+}
+#else
+#define readline_s_get_screen_size rb_f_notimplement
+#endif
+
+#ifdef HAVE_RL_VI_EDITING_MODE
 /*
  * call-seq:
  *   Readline.vi_editing_mode -> nil
@@ -444,16 +677,15 @@ readline_attempted_completion_function(const char *text, int start, int end)
 static VALUE
 readline_s_vi_editing_mode(VALUE self)
 {
-#ifdef HAVE_RL_VI_EDITING_MODE
     rb_secure(4);
     rl_vi_editing_mode(1,0);
     return Qnil;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_VI_EDITING_MODE */
 }
+#else
+#define readline_s_vi_editing_mode rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_EDITING_MODE
 /*
  * call-seq:
  *   Readline.vi_editing_mode? -> bool
@@ -467,15 +699,14 @@ readline_s_vi_editing_mode(VALUE self)
 static VALUE
 readline_s_vi_editing_mode_p(VALUE self)
 {
-#ifdef HAVE_RL_EDITING_MODE
     rb_secure(4);
     return rl_editing_mode == 0 ? Qtrue : Qfalse;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_EDITING_MODE */
 }
+#else
+#define readline_s_vi_editing_mode_p rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_EMACS_EDITING_MODE
 /*
  * call-seq:
  *   Readline.emacs_editing_mode -> nil
@@ -490,16 +721,15 @@ readline_s_vi_editing_mode_p(VALUE self)
 static VALUE
 readline_s_emacs_editing_mode(VALUE self)
 {
-#ifdef HAVE_RL_EMACS_EDITING_MODE
     rb_secure(4);
     rl_emacs_editing_mode(1,0);
     return Qnil;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_EMACS_EDITING_MODE */
 }
+#else
+#define readline_s_emacs_editing_mode rb_f_notimplement
+#endif
 
+#ifdef  HAVE_RL_EDITING_MODE
 /*
  * call-seq:
  *   Readline.emacs_editing_mode? -> bool
@@ -513,15 +743,14 @@ readline_s_emacs_editing_mode(VALUE self)
 static VALUE
 readline_s_emacs_editing_mode_p(VALUE self)
 {
-#ifdef  HAVE_RL_EDITING_MODE
     rb_secure(4);
     return rl_editing_mode == 1 ? Qtrue : Qfalse;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_EDITING_MODE */
 }
+#else
+#define readline_s_emacs_editing_mode_p rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
 /*
  * call-seq:
  *   Readline.completion_append_character = char
@@ -532,20 +761,20 @@ readline_s_emacs_editing_mode_p(VALUE self)
  *
  * For example:
  *   require "readline"
- *   
+ *
  *   Readline.readline("> ", true)
  *   Readline.completion_append_character = " "
  *
  * Result:
  *   >
  *   Input "/var/li".
- *   
+ *
  *   > /var/li
  *   Press TAB key.
- *   
+ *
  *   > /var/lib
  *   Completes "b" and appends " ". So, you can continuously input "/usr".
- *   
+ *
  *   > /var/lib /usr
  *
  * NOTE: Only one character can be specified. When "string" is
@@ -563,13 +792,12 @@ readline_s_emacs_editing_mode_p(VALUE self)
 static VALUE
 readline_s_set_completion_append_character(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
     rb_secure(4);
     if (NIL_P(str)) {
 	rl_completion_append_character = '\0';
     }
     else {
-	SafeStringValue(str);
+	OutputStringValue(str);
 	if (RSTRING_LEN(str) == 0) {
 	    rl_completion_append_character = '\0';
 	} else {
@@ -577,12 +805,12 @@ readline_s_set_completion_append_character(VALUE self, VALUE str)
 	}
     }
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETION_APPEND_CHARACTER */
 }
+#else
+#define readline_s_set_completion_append_character rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
 /*
  * call-seq:
  *   Readline.completion_append_character -> char
@@ -597,22 +825,20 @@ readline_s_set_completion_append_character(VALUE self, VALUE str)
 static VALUE
 readline_s_get_completion_append_character(VALUE self)
 {
-#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
-    VALUE str;
+    char buf[1];
 
     rb_secure(4);
     if (rl_completion_append_character == '\0')
 	return Qnil;
 
-    str = rb_str_new("", 1);
-    RSTRING_PTR(str)[0] = rl_completion_append_character;
-    return str;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETION_APPEND_CHARACTER */
+    buf[0] = (char) rl_completion_append_character;
+    return rb_locale_str_new(buf, 1);
 }
+#else
+#define readline_s_get_completion_append_character rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
 /*
  * call-seq:
  *   Readline.basic_word_break_characters = string
@@ -628,11 +854,10 @@ readline_s_get_completion_append_character(VALUE self)
 static VALUE
 readline_s_set_basic_word_break_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
     static char *basic_word_break_characters = NULL;
 
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (basic_word_break_characters == NULL) {
 	basic_word_break_characters =
 	    ALLOC_N(char, RSTRING_LEN(str) + 1);
@@ -645,19 +870,19 @@ readline_s_set_basic_word_break_characters(VALUE self, VALUE str)
     basic_word_break_characters[RSTRING_LEN(str)] = '\0';
     rl_basic_word_break_characters = basic_word_break_characters;
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_BASIC_WORD_BREAK_CHARACTERS */
 }
+#else
+#define readline_s_set_basic_word_break_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
 /*
  * call-seq:
  *   Readline.basic_word_break_characters -> string
  *
  * Gets the basic list of characters that signal a break between words
  * for the completer routine.
- * 
+ *
  * Raises NotImplementedError if the using readline library does not support.
  *
  * Raises SecurityError exception if $SAFE is 4.
@@ -665,17 +890,16 @@ readline_s_set_basic_word_break_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_get_basic_word_break_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
     rb_secure(4);
     if (rl_basic_word_break_characters == NULL)
 	return Qnil;
-    return rb_tainted_str_new2(rl_basic_word_break_characters);
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_BASIC_WORD_BREAK_CHARACTERS */
+    return rb_locale_str_new_cstr(rl_basic_word_break_characters);
 }
+#else
+#define readline_s_get_basic_word_break_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
 /*
  * call-seq:
  *   Readline.completer_word_break_characters = string
@@ -691,11 +915,10 @@ readline_s_get_basic_word_break_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_set_completer_word_break_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
     static char *completer_word_break_characters = NULL;
 
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (completer_word_break_characters == NULL) {
 	completer_word_break_characters =
 	    ALLOC_N(char, RSTRING_LEN(str) + 1);
@@ -708,19 +931,19 @@ readline_s_set_completer_word_break_characters(VALUE self, VALUE str)
     completer_word_break_characters[RSTRING_LEN(str)] = '\0';
     rl_completer_word_break_characters = completer_word_break_characters;
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS */
 }
+#else
+#define readline_s_set_completer_word_break_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
 /*
  * call-seq:
  *   Readline.completer_word_break_characters -> string
  *
  * Gets the basic list of characters that signal a break between words
  * for rl_complete_internal().
- * 
+ *
  * Raises NotImplementedError if the using readline library does not support.
  *
  * Raises SecurityError exception if $SAFE is 4.
@@ -728,17 +951,16 @@ readline_s_set_completer_word_break_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_get_completer_word_break_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
     rb_secure(4);
     if (rl_completer_word_break_characters == NULL)
 	return Qnil;
-    return rb_tainted_str_new2(rl_completer_word_break_characters);
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS */
+    return rb_locale_str_new_cstr(rl_completer_word_break_characters);
 }
+#else
+#define readline_s_get_completer_word_break_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.basic_quote_characters = string
@@ -752,11 +974,10 @@ readline_s_get_completer_word_break_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_set_basic_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
     static char *basic_quote_characters = NULL;
 
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (basic_quote_characters == NULL) {
 	basic_quote_characters =
 	    ALLOC_N(char, RSTRING_LEN(str) + 1);
@@ -770,12 +991,12 @@ readline_s_set_basic_quote_characters(VALUE self, VALUE str)
     rl_basic_quote_characters = basic_quote_characters;
 
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_BASIC_QUOTE_CHARACTERS */
 }
+#else
+#define readline_s_set_basic_quote_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.basic_quote_characters -> string
@@ -789,17 +1010,16 @@ readline_s_set_basic_quote_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_get_basic_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
     rb_secure(4);
     if (rl_basic_quote_characters == NULL)
 	return Qnil;
-    return rb_tainted_str_new2(rl_basic_quote_characters);
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_BASIC_QUOTE_CHARACTERS */
+    return rb_locale_str_new_cstr(rl_basic_quote_characters);
 }
+#else
+#define readline_s_get_basic_quote_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.completer_quote_characters = string
@@ -816,11 +1036,10 @@ readline_s_get_basic_quote_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_set_completer_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
     static char *completer_quote_characters = NULL;
 
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (completer_quote_characters == NULL) {
 	completer_quote_characters =
 	    ALLOC_N(char, RSTRING_LEN(str) + 1);
@@ -833,12 +1052,12 @@ readline_s_set_completer_quote_characters(VALUE self, VALUE str)
     rl_completer_quote_characters = completer_quote_characters;
 
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETER_QUOTE_CHARACTERS */
 }
+#else
+#define readline_s_set_completer_quote_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.completer_quote_characters -> string
@@ -853,17 +1072,16 @@ readline_s_set_completer_quote_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_get_completer_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
     rb_secure(4);
     if (rl_completer_quote_characters == NULL)
 	return Qnil;
-    return rb_tainted_str_new2(rl_completer_quote_characters);
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_COMPLETER_QUOTE_CHARACTERS */
+    return rb_locale_str_new_cstr(rl_completer_quote_characters);
 }
+#else
+#define readline_s_get_completer_quote_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.filename_quote_characters = string
@@ -878,11 +1096,10 @@ readline_s_get_completer_quote_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_set_filename_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
     static char *filename_quote_characters = NULL;
 
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (filename_quote_characters == NULL) {
 	filename_quote_characters =
 	    ALLOC_N(char, RSTRING_LEN(str) + 1);
@@ -895,12 +1112,12 @@ readline_s_set_filename_quote_characters(VALUE self, VALUE str)
     rl_filename_quote_characters = filename_quote_characters;
 
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_FILENAME_QUOTE_CHARACTERS */
 }
+#else
+#define readline_s_set_filename_quote_characters rb_f_notimplement
+#endif
 
+#ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
 /*
  * call-seq:
  *   Readline.filename_quote_characters -> string
@@ -915,21 +1132,39 @@ readline_s_set_filename_quote_characters(VALUE self, VALUE str)
 static VALUE
 readline_s_get_filename_quote_characters(VALUE self, VALUE str)
 {
-#ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
     rb_secure(4);
     if (rl_filename_quote_characters == NULL)
 	return Qnil;
-    return rb_tainted_str_new2(rl_filename_quote_characters);
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif /* HAVE_RL_FILENAME_QUOTE_CHARACTERS */
+    return rb_locale_str_new_cstr(rl_filename_quote_characters);
 }
+#else
+#define readline_s_get_filename_quote_characters rb_f_notimplement
+#endif
+
+#ifdef HAVE_RL_REFRESH_LINE
+/*
+ * call-seq:
+ *   Readline.refresh_line -> nil
+ *
+ * Clear the current input line.
+ *
+ * Raises SecurityError exception if $SAFE is 4.
+ */
+static VALUE
+readline_s_refresh_line(VALUE self)
+{
+    rb_secure(4);
+    rl_refresh_line(0, 0);
+    return Qnil;
+}
+#else
+#define readline_s_refresh_line rb_f_notimplement
+#endif
 
 static VALUE
 hist_to_s(VALUE self)
 {
-    return rb_str_new2("HISTORY");
+    return rb_str_new_cstr("HISTORY");
 }
 
 static int
@@ -961,19 +1196,19 @@ hist_get(VALUE self, VALUE index)
     if (entry == NULL) {
 	rb_raise(rb_eIndexError, "invalid index");
     }
-    return rb_tainted_str_new2(entry->line);
+    return rb_locale_str_new_cstr(entry->line);
 }
 
+#ifdef HAVE_REPLACE_HISTORY_ENTRY
 static VALUE
 hist_set(VALUE self, VALUE index, VALUE str)
 {
-#ifdef HAVE_REPLACE_HISTORY_ENTRY
     HIST_ENTRY *entry = NULL;
     int i;
 
     rb_secure(4);
     i = NUM2INT(index);
-    SafeStringValue(str);
+    OutputStringValue(str);
     if (i < 0) {
         i += history_length;
     }
@@ -984,17 +1219,16 @@ hist_set(VALUE self, VALUE index, VALUE str)
 	rb_raise(rb_eIndexError, "invalid index");
     }
     return str;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif
 }
+#else
+#define hist_set rb_f_notimplement
+#endif
 
 static VALUE
 hist_push(VALUE self, VALUE str)
 {
     rb_secure(4);
-    SafeStringValue(str);
+    OutputStringValue(str);
     add_history(RSTRING_PTR(str));
     return self;
 }
@@ -1003,11 +1237,11 @@ static VALUE
 hist_push_method(int argc, VALUE *argv, VALUE self)
 {
     VALUE str;
-    
+
     rb_secure(4);
     while (argc--) {
 	str = *argv++;
-	SafeStringValue(str);
+	OutputStringValue(str);
 	add_history(RSTRING_PTR(str));
     }
     return self;
@@ -1023,7 +1257,7 @@ rb_remove_history(int index)
     rb_secure(4);
     entry = remove_history(index);
     if (entry) {
-        val = rb_tainted_str_new2(entry->line);
+        val = rb_locale_str_new_cstr(entry->line);
         free((void *) entry->line);
         free(entry);
         return val;
@@ -1070,7 +1304,7 @@ hist_each(VALUE self)
         entry = history_get(history_get_offset_func(i));
         if (entry == NULL)
             break;
-	rb_yield(rb_tainted_str_new2(entry->line));
+	rb_yield(rb_locale_str_new_cstr(entry->line));
     }
     return self;
 }
@@ -1104,18 +1338,17 @@ hist_delete_at(VALUE self, VALUE index)
     return rb_remove_history(i);
 }
 
+#ifdef HAVE_CLEAR_HISTORY
 static VALUE
 hist_clear(VALUE self)
 {
-#ifdef HAVE_CLEAR_HISTORY
     rb_secure(4);
     clear_history();
     return self;
-#else
-    rb_notimplement();
-    return Qnil; /* not reached */
-#endif
 }
+#else
+#define hist_clear rb_f_notimplement
+#endif
 
 static VALUE
 filename_completion_proc_call(VALUE self, VALUE str)
@@ -1129,7 +1362,7 @@ filename_completion_proc_call(VALUE self, VALUE str)
     if (matches) {
 	result = rb_ary_new();
 	for (i = 0; matches[i]; i++) {
-	    rb_ary_push(result, rb_tainted_str_new2(matches[i]));
+	    rb_ary_push(result, rb_locale_str_new_cstr(matches[i]));
 	    free(matches[i]);
 	}
 	free(matches);
@@ -1154,7 +1387,7 @@ username_completion_proc_call(VALUE self, VALUE str)
     if (matches) {
 	result = rb_ary_new();
 	for (i = 0; matches[i]; i++) {
-	    rb_ary_push(result, rb_tainted_str_new2(matches[i]));
+	    rb_ary_push(result, rb_locale_str_new_cstr(matches[i]));
 	    free(matches[i]);
 	}
 	free(matches);
@@ -1173,7 +1406,7 @@ Init_readline()
     VALUE history, fcomp, ucomp, version;
 
     /* Allow conditional parsing of the ~/.inputrc file. */
-    rl_readline_name = "Ruby";
+    rl_readline_name = (char *)"Ruby";
 
     using_history();
 
@@ -1195,6 +1428,14 @@ Init_readline()
 			       readline_s_set_completion_case_fold, 1);
     rb_define_singleton_method(mReadline, "completion_case_fold",
 			       readline_s_get_completion_case_fold, 0);
+    rb_define_singleton_method(mReadline, "line_buffer",
+			       readline_s_get_line_buffer, 0);
+    rb_define_singleton_method(mReadline, "point",
+			       readline_s_get_point, 0);
+    rb_define_singleton_method(mReadline, "set_screen_size",
+			       readline_s_set_screen_size, 2);
+    rb_define_singleton_method(mReadline, "get_screen_size",
+			       readline_s_get_screen_size, 0);
     rb_define_singleton_method(mReadline, "vi_editing_mode",
 			       readline_s_vi_editing_mode, 0);
     rb_define_singleton_method(mReadline, "vi_editing_mode?",
@@ -1227,6 +1468,13 @@ Init_readline()
 			       readline_s_set_filename_quote_characters, 1);
     rb_define_singleton_method(mReadline, "filename_quote_characters",
 			       readline_s_get_filename_quote_characters, 0);
+    rb_define_singleton_method(mReadline, "refresh_line",
+			       readline_s_refresh_line, 0);
+
+#if USE_INSERT_IGNORE_ESCAPE
+    CONST_ID(id_orig_prompt, "orig_prompt");
+    CONST_ID(id_last_prompt, "last_prompt");
+#endif
 
     history = rb_obj_alloc(rb_cObject);
     rb_extend_object(history, rb_mEnumerable);
@@ -1271,9 +1519,9 @@ Init_readline()
     rb_define_const(mReadline, "USERNAME_COMPLETION_PROC", ucomp);
     history_get_offset_func = history_get_offset_history_base;
 #if defined HAVE_RL_LIBRARY_VERSION
-    version = rb_str_new2(rl_library_version);
+    version = rb_str_new_cstr(rl_library_version);
 #if defined HAVE_CLEAR_HISTORY || defined HAVE_REMOVE_HISTORY
-    if (strncmp(rl_library_version, EDIT_LINE_LIBRARY_VERSION, 
+    if (strncmp(rl_library_version, EDIT_LINE_LIBRARY_VERSION,
 		strlen(EDIT_LINE_LIBRARY_VERSION)) == 0) {
 	add_history("1");
 	if (history_get(history_get_offset_func(0)) == NULL) {
@@ -1284,23 +1532,36 @@ Init_readline()
 #else
 	{
 	    HIST_ENTRY *entry = remove_history(0);
-	    free(entry->line);
-	    free(entry);
+	    if (entry) {
+		free((char *)entry->line);
+		free(entry);
+	    }
 	}
 #endif
     }
 #endif
 #else
-    version = rb_str_new2("2.0 or prior version");
+    version = rb_str_new_cstr("2.0 or prior version");
 #endif
     /* Version string of GNU Readline or libedit. */
     rb_define_const(mReadline, "VERSION", version);
 
     rl_attempted_completion_function = readline_attempted_completion_function;
-#ifdef HAVE_RL_EVENT_HOOK
+#if defined HAVE_RL_GETC_FUNCTION
+    rl_getc_function = readline_getc;
+    id_getbyte = rb_intern_const("getbyte");
+#elif defined HAVE_RL_EVENT_HOOK
     rl_event_hook = readline_event;
 #endif
+#ifdef HAVE_RL_CATCH_SIGNALS
+    rl_catch_signals = 0;
+#endif
+#ifdef HAVE_RL_CATCH_SIGWINCH
+    rl_catch_sigwinch = 0;
+#endif
 #ifdef HAVE_RL_CLEAR_SIGNALS
     rl_clear_signals();
 #endif
+
+    readline_s_set_input(mReadline, rb_stdin);
 }
