@@ -2,7 +2,7 @@
 /*
  * rubyext.c
  *
- * $Author: nobu $
+ * $Author: yugui $
  *
  * Copyright (C) 2003-2005 why the lucky stiff
  */
@@ -50,11 +50,11 @@ typedef struct {
  * symbols and constants
  */
 static ID s_new, s_utc, s_at, s_to_f, s_to_i, s_read, s_binmode, s_call, s_cmp, s_transfer, s_update, s_dup, s_haskey, s_match, s_keys, s_unpack, s_tr_bang, s_default_set, s_tag_read_class, s_tag_subclasses, s_resolver, s_push, s_emitter, s_level, s_detect_implicit, s_node_import, s_out, s_input, s_intern, s_transform, s_yaml_new, s_yaml_initialize, s_node_export, s_to_yaml, s_write, s_set_resolver, s_each;
-static ID s_tags, s_kind, s_name, s_options, s_type_id, s_type_id_set, s_style, s_style_set, s_value, s_value_set;
+static ID s_tags, s_kind, s_name, s_options, s_type_id, s_type_id_set, s_style, s_style_set, s_value, s_value_set, s_parse;
 static VALUE sym_model, sym_generic, sym_input, sym_bytecode;
 static VALUE sym_scalar, sym_seq, sym_map;
 static VALUE sym_1quote, sym_2quote, sym_fold, sym_literal, sym_plain, sym_inline;
-static VALUE cDate, cNode, cMap, cSeq, cScalar, cOut, cParser, cResolver, cPrivateType, cDomainType, cYObject, cBadAlias, cDefaultKey, cMergeKey, cEmitter;
+static VALUE cDate, cNode, cMap, cSeq, cScalar, cOut, cParser, cResolver, cPrivateType, cDomainType, cYObject, cBadAlias, cDefaultKey, cMergeKey, cEmitter, cDateTime;
 static VALUE oDefaultResolver, oGenericResolver;
 
 /*
@@ -196,7 +196,7 @@ VALUE
 syck_get_hash_aref(VALUE hsh, VALUE key)
 {
    VALUE val = rb_hash_aref( hsh, key );
-   if ( NIL_P( val ) ) 
+   if ( NIL_P( val ) )
    {
        val = rb_hash_new();
        rb_hash_aset(hsh, key, val);
@@ -207,10 +207,17 @@ syck_get_hash_aref(VALUE hsh, VALUE key)
 /*
  * creating timestamps
  */
+struct mktime_arg {
+    char *str;
+    long len;
+};
+
 SYMID
-rb_syck_mktime(char *str, long len)
+mktime_do(struct mktime_arg *arg)
 {
     VALUE time;
+    char *str = arg->str;
+    long len = arg->len;
     char *ptr = str;
     VALUE year = INT2FIX(0);
     VALUE mon = INT2FIX(0);
@@ -218,7 +225,7 @@ rb_syck_mktime(char *str, long len)
     VALUE hour = INT2FIX(0);
     VALUE min = INT2FIX(0);
     VALUE sec = INT2FIX(0);
-    long usec;
+    double usec;
 
     /* Year*/
     if ( ptr[0] != '\0' && len > 0 ) {
@@ -264,19 +271,20 @@ rb_syck_mktime(char *str, long len)
     ptr += 2;
     if ( len > ptr - str && *ptr == '.' )
     {
-        char padded[] = "000000";
-        char *end = ptr + 1;
-        char *p = end;
+        char padded[] = "000000.000000";
+        const int padding = 6;
+        const int offset = padding + 1;
+        const char *end = ptr + 1;
+        const char *begin = end;
+        int length;
         while ( isdigit( *end ) ) end++;
-        if (end - p < sizeof(padded)) {
-            MEMCPY(padded, ptr + 1, char, end - (ptr + 1));
-            p = padded;
-        }
-        usec = strtol(p, NULL, 10);
+        length = (int)(end - begin) <= padding ? (int)(end - begin) : padding;
+        MEMCPY(padded, begin, char, length);
+        usec = strtod(padded, NULL);
     }
     else
     {
-        usec = 0;
+        usec = 0.0;
     }
 
     /* Time Zone*/
@@ -284,7 +292,7 @@ rb_syck_mktime(char *str, long len)
     if ( len > ptr - str && ( *ptr == '-' || *ptr == '+' ) )
     {
         time_t tz_offset = strtol(ptr, NULL, 10) * 3600;
-        time_t tmp;
+        VALUE tmp;
 
         while ( *ptr != ':' && *ptr != '\0' ) ptr++;
         if ( *ptr == ':' )
@@ -302,16 +310,40 @@ rb_syck_mktime(char *str, long len)
 
         /* Make TZ time*/
         time = rb_funcall(rb_cTime, s_utc, 6, year, mon, day, hour, min, sec);
-        tmp = NUM2LONG(rb_funcall(time, s_to_i, 0)) - tz_offset;
-        return rb_funcall(rb_cTime, s_at, 2, LONG2NUM(tmp), LONG2NUM(usec));
+        tmp = rb_funcall(time, s_to_i, 0);
+        tmp = rb_funcall(tmp, '-', 1, LONG2FIX(tz_offset));
+        return rb_funcall(rb_cTime, s_at, 2, tmp, rb_float_new(usec));
     }
     else
     {
         /* Make UTC time*/
-        return rb_funcall(rb_cTime, s_utc, 7, year, mon, day, hour, min, sec, LONG2NUM(usec));
+        return rb_funcall(rb_cTime, s_utc, 7, year, mon, day, hour, min, sec, rb_float_new(usec));
     }
 }
 
+SYMID
+mktime_r(struct mktime_arg *arg)
+{
+    if (!cDateTime) {
+        /*
+         * Load Date module
+         */
+        rb_require("date");
+        cDateTime = rb_const_get(rb_cObject, rb_intern("DateTime"));
+    }
+    return rb_funcall(cDateTime, s_parse, 1, rb_str_new(arg->str, arg->len));
+}
+
+SYMID
+rb_syck_mktime(char *str, long len)
+{
+    struct mktime_arg a;
+
+    a.str = str;
+    a.len = len;
+    return rb_rescue2(mktime_do, (VALUE)&a, mktime_r, (VALUE)&a, rb_eArgError, NULL);
+}
+
 /*
  * handles merging of an array of hashes
  * (see http://www.yaml.org/type/merge/)
@@ -506,11 +538,11 @@ yaml_org_handler( SyckNode *n, VALUE *ref )
                 obj = rb_funcall( cDefaultKey, s_new, 0 );
             }
             else if ( n->data.str->style == scalar_plain &&
-                      n->data.str->len > 1 && 
+                      n->data.str->len > 1 &&
                       strncmp( n->data.str->ptr, ":", 1 ) == 0 )
             {
-                obj = rb_funcall( oDefaultResolver, s_transfer, 2, 
-                                  rb_str_new2( "tag:ruby.yaml.org,2002:sym" ), 
+                obj = rb_funcall( oDefaultResolver, s_transfer, 2,
+                                  rb_str_new2( "tag:ruby.yaml.org,2002:sym" ),
                                   rb_str_new( n->data.str->ptr + 1, n->data.str->len - 1 ) );
             }
             else if ( strcmp( type_id, "str" ) == 0 )
@@ -613,7 +645,7 @@ rb_syck_load_handler(SyckParser *p, SyckNode *n)
     }
 
     /*
-     * Create node, 
+     * Create node,
      */
     obj = rb_funcall( resolver, s_node_import, 1, Data_Wrap_Struct( cNode, NULL, NULL, n ) );
 
@@ -649,8 +681,8 @@ rb_syck_err_handler(SyckParser *p, const char *msg)
     rb_raise(rb_eArgError, "%s on line %d, col %"PRIdPTRDIFF": `%s'",
            msg,
            p->linect,
-           p->cursor - p->lineptr, 
-           p->lineptr); 
+           p->cursor - p->lineptr,
+           p->lineptr);
 }
 
 /*
@@ -684,7 +716,7 @@ syck_set_model(VALUE p, VALUE input, VALUE model)
 
     if ( NIL_P( input ) )
     {
-        input = rb_ivar_get( p, s_input ); 
+        input = rb_ivar_get( p, s_input );
     }
     if ( input == sym_bytecode )
     {
@@ -742,7 +774,7 @@ rb_syck_free_parser(SyckParser *p)
  * YAML::Syck::Parser.allocate
  */
 VALUE syck_parser_s_alloc _((VALUE));
-VALUE 
+VALUE
 syck_parser_s_alloc(VALUE class)
 {
     VALUE pobj;
@@ -849,7 +881,7 @@ syck_parser_load_documents(int argc, VALUE *argv, VALUE self)
     model = rb_hash_aref( rb_attr_get( self, s_options ), sym_model );
     Data_Get_Struct(self, SyckParser, parser);
     syck_set_model( self, input, model );
-    
+
     bonus = (struct parser_xtra *)parser->bonus;
     bonus->taint = syck_parser_assign_io(parser, &port);
     bonus->resolver = rb_attr_get( self, s_resolver );
@@ -916,7 +948,7 @@ syck_resolver_use_types_at(VALUE self, VALUE hsh)
 }
 
 /*
- * YAML::Syck::Resolver#detect_implicit 
+ * YAML::Syck::Resolver#detect_implicit
  */
 VALUE
 syck_resolver_detect_implicit(VALUE self, VALUE val)
@@ -1044,7 +1076,7 @@ syck_const_find(VALUE const_name)
 VALUE
 syck_resolver_transfer(VALUE self, VALUE type, VALUE val)
 {
-    if (NIL_P(type) || RSTRING_LEN(StringValue(type)) == 0) 
+    if (NIL_P(type) || RSTRING_LEN(StringValue(type)) == 0)
     {
         type = rb_funcall( self, s_detect_implicit, 1, val );
     }
@@ -1092,7 +1124,7 @@ syck_resolver_transfer(VALUE self, VALUE type, VALUE val)
                         subclass = rb_funcall( target_class, s_tag_read_class, 1, subclass );
                         subclass_v = syck_const_find( subclass );
 
-                        if ( subclass_v != Qnil ) 
+                        if ( subclass_v != Qnil )
                         {
                             subclass = subclass_v;
                         }
@@ -1146,7 +1178,7 @@ syck_resolver_transfer(VALUE self, VALUE type, VALUE val)
                     rb_block_call( val, s_each, 0, 0, syck_set_ivars, obj );
                 }
             }
-            else 
+            else
             {
                 VALUE parts = rb_str_split( type, ":" );
                 VALUE scheme = rb_ary_shift( parts );
@@ -1188,7 +1220,7 @@ syck_resolver_tagurize(VALUE self, VALUE val)
 }
 
 /*
- * YAML::Syck::DefaultResolver#detect_implicit 
+ * YAML::Syck::DefaultResolver#detect_implicit
  */
 VALUE
 syck_defaultresolver_detect_implicit(VALUE self, VALUE val)
@@ -1247,19 +1279,19 @@ syck_genericresolver_node_import(VALUE self, VALUE node)
             if ( n->data.str->style == scalar_1quote )
             {
                 style = sym_1quote;
-            } 
+            }
             else if ( n->data.str->style == scalar_2quote )
             {
                 style = sym_2quote;
-            } 
+            }
             else if ( n->data.str->style == scalar_fold )
             {
                 style = sym_fold;
-            } 
+            }
             else if ( n->data.str->style == scalar_literal )
             {
                 style = sym_literal;
-            } 
+            }
             else if ( n->data.str->style == scalar_plain )
             {
                 style = sym_plain;
@@ -1277,7 +1309,7 @@ syck_genericresolver_node_import(VALUE self, VALUE node)
             if ( n->data.list->style == seq_inline )
             {
                 style = sym_inline;
-            } 
+            }
             obj = rb_funcall( cSeq, s_new, 3, t, v, style );
             rb_iv_set(obj, "@kind", sym_seq);
         break;
@@ -1291,7 +1323,7 @@ syck_genericresolver_node_import(VALUE self, VALUE node)
             if ( n->data.pairs->style == map_inline )
             {
                 style = sym_inline;
-            } 
+            }
             obj = rb_funcall( cMap, s_new, 3, t, v, style );
             rb_iv_set(obj, "@kind", sym_map);
         break;
@@ -1316,8 +1348,8 @@ syck_badalias_initialize(VALUE self, VALUE val)
 VALUE
 syck_badalias_cmp(VALUE alias1, VALUE alias2)
 {
-    VALUE str1 = rb_ivar_get( alias1, s_name ); 
-    VALUE str2 = rb_ivar_get( alias2, s_name ); 
+    VALUE str1 = rb_ivar_get( alias1, s_name );
+    VALUE str2 = rb_ivar_get( alias2, s_name );
     VALUE val = rb_funcall( str1, s_cmp, 1, str2 );
     return val;
 }
@@ -1428,23 +1460,23 @@ syck_scalar_style_set(VALUE self, VALUE style)
     if ( NIL_P( style ) )
     {
         node->data.str->style = scalar_none;
-    } 
+    }
     else if ( style == sym_1quote )
     {
         node->data.str->style = scalar_1quote;
-    } 
+    }
     else if ( style == sym_2quote )
     {
         node->data.str->style = scalar_2quote;
-    } 
+    }
     else if ( style == sym_fold )
     {
         node->data.str->style = scalar_fold;
-    } 
+    }
     else if ( style == sym_literal )
     {
         node->data.str->style = scalar_literal;
-    } 
+    }
     else if ( style == sym_plain )
     {
         node->data.str->style = scalar_plain;
@@ -1556,7 +1588,7 @@ syck_seq_style_set(VALUE self, VALUE style)
     if ( style == sym_inline )
     {
         node->data.list->style = seq_inline;
-    } 
+    }
     else
     {
         node->data.list->style = seq_none;
@@ -1678,7 +1710,7 @@ syck_map_style_set(VALUE self, VALUE style)
     if ( style == sym_inline )
     {
         node->data.pairs->style = map_inline;
-    } 
+    }
     else
     {
         node->data.pairs->style = map_none;
@@ -1834,7 +1866,7 @@ rb_syck_emitter_handler(SyckEmitter *e, st_data_t data)
 /*
  * Handle output from the emitter
  */
-void 
+void
 rb_syck_output_handler(SyckEmitter * emitter, char *str, long len)
 {
     struct emitter_xtra *bonus = (struct emitter_xtra *)emitter->bonus;
@@ -1890,7 +1922,7 @@ rb_syck_free_emitter(SyckEmitter *e)
  * YAML::Syck::Emitter.allocate
  */
 VALUE syck_emitter_s_alloc _((VALUE));
-VALUE 
+VALUE
 syck_emitter_s_alloc(VALUE class)
 {
     VALUE pobj;
@@ -1907,6 +1939,15 @@ syck_emitter_s_alloc(VALUE class)
     return pobj;
 }
 
+static VALUE
+id_hash_new(void)
+{
+    VALUE hash;
+    hash = rb_hash_new();
+    rb_funcall(hash, rb_intern("compare_by_identity"), 0);
+    return hash;
+}
+
 /*
  * YAML::Syck::Emitter.reset( options )
  */
@@ -1922,7 +1963,7 @@ syck_emitter_reset(int argc, VALUE *argv, VALUE self)
 
     bonus->oid = Qnil;
     bonus->port = rb_str_new2( "" );
-    bonus->data = rb_hash_new();
+    bonus->data = id_hash_new();
 
     if (rb_scan_args(argc, argv, "01", &options) == 0)
     {
@@ -1942,7 +1983,7 @@ syck_emitter_reset(int argc, VALUE *argv, VALUE self)
         Check_Type(options, T_HASH);
         rb_ivar_set(self, s_options, options);
     }
-    
+
     emitter->headless = 0;
     rb_ivar_set(self, s_level, INT2FIX(0));
     rb_ivar_set(self, s_resolver, Qnil);
@@ -1978,14 +2019,14 @@ syck_emitter_emit(int argc, VALUE *argv, VALUE self)
     /* Second pass, build emitted string */
     level -= 1;
     rb_ivar_set(self, s_level, INT2FIX(level));
-    if ( level == 0 ) 
+    if ( level == 0 )
     {
         syck_emit(emitter, (st_data_t)symple);
         syck_emitter_flush(emitter, 0);
 
         return bonus->port;
     }
-    
+
     return symple;
 }
 
@@ -2071,9 +2112,7 @@ syck_out_scalar(int argc, VALUE *argv, VALUE self)
 void
 Init_syck()
 {
-    VALUE rb_yaml = rb_define_module( "YAML" );
-    VALUE rb_syck = rb_define_module_under( rb_yaml, "Syck" );
-    rb_define_const( rb_syck, "VERSION", rb_str_new2( SYCK_VERSION ) );
+    VALUE rb_syck = rb_define_module_under( rb_cObject, "Syck" );
     rb_define_module_function( rb_syck, "compile", rb_syck_compile, 1 );
 
     /*
@@ -2112,6 +2151,7 @@ Init_syck()
     s_yaml_new = rb_intern("yaml_new");
     s_yaml_initialize = rb_intern("yaml_initialize");
     s_each = rb_intern("each");
+    s_parse = rb_intern("parse");
 
     s_tags = rb_intern("@tags");
     s_name = rb_intern("@name");
@@ -2217,7 +2257,7 @@ Init_syck()
     /*
      * Define YAML::PrivateType class
      */
-    cPrivateType = rb_define_class_under( rb_yaml, "PrivateType", rb_cObject );
+    cPrivateType = rb_define_class_under( rb_syck, "PrivateType", rb_cObject );
     rb_define_attr( cPrivateType, "type_id", 1, 1 );
     rb_define_attr( cPrivateType, "value", 1, 1 );
     rb_define_method( cPrivateType, "initialize", syck_privatetype_initialize, 2);
@@ -2225,7 +2265,7 @@ Init_syck()
     /*
      * Define YAML::DomainType class
      */
-    cDomainType = rb_define_class_under( rb_yaml, "DomainType", rb_cObject );
+    cDomainType = rb_define_class_under( rb_syck, "DomainType", rb_cObject );
     rb_define_attr( cDomainType, "domain", 1, 1 );
     rb_define_attr( cDomainType, "type_id", 1, 1 );
     rb_define_attr( cDomainType, "value", 1, 1 );
@@ -2234,7 +2274,7 @@ Init_syck()
     /*
      * Define YAML::Object class
      */
-    cYObject = rb_define_class_under( rb_yaml, "Object", rb_cObject );
+    cYObject = rb_define_class_under( rb_syck, "Object", rb_cObject );
     rb_define_attr( cYObject, "class", 1, 1 );
     rb_define_attr( cYObject, "ivars", 1, 1 );
     rb_define_method( cYObject, "initialize", syck_yobject_initialize, 2);
