@@ -1,10 +1,10 @@
-/* 
+/*
  * UNIX Syslog extension for Ruby
  * Amos Gouaux, University of Texas at Dallas
  * <amos+ruby@utdallas.edu>
  *
  * $RoughId: syslog.c,v 1.21 2002/02/25 12:21:17 knu Exp $
- * $Id: syslog.c 12501 2007-06-10 03:06:15Z nobu $
+ * $Id: syslog.c 35742 2012-05-21 07:28:54Z knu $
  */
 
 #include "ruby/ruby.h"
@@ -22,6 +22,7 @@ static void syslog_write(int pri, int argc, VALUE *argv)
 {
     VALUE str;
 
+    rb_secure(4);
     if (argc < 1) {
         rb_raise(rb_eArgError, "no log message supplied");
     }
@@ -38,6 +39,7 @@ static void syslog_write(int pri, int argc, VALUE *argv)
 /* Syslog module methods */
 static VALUE mSyslog_close(VALUE self)
 {
+    rb_secure(4);
     if (!syslog_opened) {
         rb_raise(rb_eRuntimeError, "syslog not opened");
     }
@@ -63,13 +65,9 @@ static VALUE mSyslog_open(int argc, VALUE *argv, VALUE self)
     rb_scan_args(argc, argv, "03", &ident, &opt, &fac);
 
     if (NIL_P(ident)) {
-        ident = rb_gv_get("$0"); 
+        ident = rb_gv_get("$0");
     }
-#ifdef SafeStringValue
     SafeStringValue(ident);
-#else
-    Check_SafeStr(ident);
-#endif
     syslog_ident = strdup(RSTRING_PTR(ident));
 
     if (NIL_P(opt)) {
@@ -132,6 +130,7 @@ static VALUE mSyslog_get_mask(VALUE self)
 
 static VALUE mSyslog_set_mask(VALUE self, VALUE mask)
 {
+    rb_secure(4);
     if (!syslog_opened) {
         rb_raise(rb_eRuntimeError, "must open syslog before setting log mask");
     }
@@ -165,6 +164,8 @@ static VALUE mSyslog_inspect(VALUE self)
 {
     char buf[1024];
 
+    Check_Type(self, T_MODULE);
+
     if (syslog_opened) {
 	snprintf(buf, sizeof(buf),
 	  "<#%s: opened=true, ident=\"%s\", options=%d, facility=%d, mask=%d>",
@@ -221,19 +222,19 @@ define_syslog_shortcut_method(LOG_DEBUG, debug)
 
 static VALUE mSyslogConstants_LOG_MASK(VALUE klass, VALUE pri)
 {
-    return INT2FIX(LOG_MASK(FIX2INT(pri)));
+    return INT2FIX(LOG_MASK(NUM2INT(pri)));
 }
 
 static VALUE mSyslogConstants_LOG_UPTO(VALUE klass, VALUE pri)
 {
-    return INT2FIX(LOG_UPTO(FIX2INT(pri)));
+    return INT2FIX(LOG_UPTO(NUM2INT(pri)));
 }
 
 /* Init for package syslog */
 void Init_syslog()
 {
     mSyslog = rb_define_module("Syslog");
- 
+
     mSyslogConstants = rb_define_module_under(mSyslog, "Constants");
 
     rb_include_module(mSyslog, mSyslogConstants);
