@@ -2,8 +2,8 @@
 
   rubysig.h -
 
-  $Author: usa $
-  $Date: 2012-12-11 17:52:48 +0900 (Tue, 11 Dec 2012) $
+  $Author: akr $
+  $Date: 2009-10-02 12:04:37 +0000 (Fri, 02 Oct 2009) $
   created at: Wed Aug 16 01:15:38 JST 1995
 
   Copyright (C) 1993-2008 Yukihiro Matsumoto
@@ -27,10 +27,6 @@ extern "C" {
 #endif
 #endif
 
-#if defined __GNUC__ && __GNUC__ >= 4
-#pragma GCC visibility push(default)
-#endif
-
 struct rb_blocking_region_buffer;
 DEPRECATED(RUBY_EXTERN struct rb_blocking_region_buffer *rb_thread_blocking_region_begin(void));
 DEPRECATED(RUBY_EXTERN void rb_thread_blocking_region_end(struct rb_blocking_region_buffer *));
@@ -42,10 +38,6 @@ DEPRECATED(RUBY_EXTERN void rb_thread_blocking_region_end(struct rb_blocking_reg
 #define ALLOW_INTS do {CHECK_INTS;} while (0)
 #define CHECK_INTS rb_thread_check_ints()
 
-#if defined __GNUC__ && __GNUC__ >= 4
-#pragma GCC visibility pop
-#endif
-
 #if defined(__cplusplus)
 #if 0
 { /* satisfy cc-mode */
