@@ -18,7 +18,7 @@
 #
 # The Ruby standard library contains the core classes of the dRuby package.
 # However, the full package also includes access control lists and the
-# Rinda tuple-space distributed task management system, as well as a 
+# Rinda tuple-space distributed task management system, as well as a
 # large number of samples.  The full dRuby package can be downloaded from
 # the dRuby home page (see *References*).
 #
@@ -121,7 +121,7 @@ require 'drb/eq'
 # are forwarded to the local object, as described in the discussion of
 # DRbObjects.  This has semantics similar to the normal Ruby
 # pass-by-reference.
-# 
+#
 # The easiest way to signal that we want an otherwise marshallable
 # object to be passed or returned as a DRbObject reference, rather
 # than marshalled and sent as a copy, is to include the
@@ -135,7 +135,7 @@ require 'drb/eq'
 # passed back to the remote execution context to be collected, before
 # the collected values are finally returned to the local context as
 # the return value of the method invocation.
-# 
+#
 # == Examples of usage
 #
 # For more dRuby samples, see the +samples+ directory in the full
@@ -148,33 +148,33 @@ require 'drb/eq'
 # starting the server code first.
 #
 # ==== Server code
-#    
+#
 #   require 'drb/drb'
-#     
+#
 #   # The URI for the server to connect to
-#   URI="druby://localhost:8787" 
-#     
+#   URI="druby://localhost:8787"
+#
 #   class TimeServer
-#     
+#
 #     def get_current_time
 #       return Time.now
 #     end
-#     
+#
 #   end
-#     
+#
 #   # The object that handles requests on the server
 #   FRONT_OBJECT=TimeServer.new
 #
 #   $SAFE = 1   # disable eval() and friends
-#   
+#
 #   DRb.start_service(URI, FRONT_OBJECT)
 #   # Wait for the drb server thread to finish before exiting.
 #   DRb.thread.join
 #
 # ==== Client code
-#     
+#
 #   require 'drb/drb'
-#   
+#
 #   # The URI to connect to
 #   SERVER_URI="druby://localhost:8787"
 #
@@ -184,43 +184,43 @@ require 'drb/eq'
 #   # as soon as we pass a non-marshallable object as an argument
 #   # to a dRuby call.
 #   DRb.start_service
-#   
+#
 #   timeserver = DRbObject.new_with_uri(SERVER_URI)
-#   puts timeserver.get_current_time 
+#   puts timeserver.get_current_time
 #
 # === Remote objects under dRuby
 #
 # This example illustrates returning a reference to an object
 # from a dRuby call.  The Logger instances live in the server
 # process.  References to them are returned to the client process,
-# where methods can be invoked upon them.  These methods are 
+# where methods can be invoked upon them.  These methods are
 # executed in the server process.
 #
 # ==== Server code
-#   
+#
 #   require 'drb/drb'
-#   
+#
 #   URI="druby://localhost:8787"
-#   
+#
 #   class Logger
 #
 #       # Make dRuby send Logger instances as dRuby references,
 #       # not copies.
 #       include DRb::DRbUndumped
-#   
+#
 #       def initialize(n, fname)
 #           @name = n
 #           @filename = fname
 #       end
-#   
+#
 #       def log(message)
 #           File.open(@filename, "a") do |f|
 #               f.puts("#{Time.now}: #{@name}: #{message}")
 #           end
 #       end
-#   
+#
 #   end
-#   
+#
 #   # We have a central object for creating and retrieving loggers.
 #   # This retains a local reference to all loggers created.  This
 #   # is so an existing logger can be looked up by name, but also
@@ -228,12 +228,12 @@ require 'drb/eq'
 #   # reference to an object is not sufficient to prevent it being
 #   # garbage collected!
 #   class LoggerFactory
-#   
+#
 #       def initialize(bdir)
 #           @basedir = bdir
 #           @loggers = {}
 #       end
-#   
+#
 #       def get_logger(name)
 #           if !@loggers.has_key? name
 #               # make the filename safe, then declare it to be so
@@ -242,34 +242,34 @@ require 'drb/eq'
 #           end
 #           return @loggers[name]
 #       end
-#   
+#
 #   end
-#   
+#
 #   FRONT_OBJECT=LoggerFactory.new("/tmp/dlog")
 #
 #   $SAFE = 1   # disable eval() and friends
-#   
+#
 #   DRb.start_service(URI, FRONT_OBJECT)
 #   DRb.thread.join
 #
 # ==== Client code
 #
 #   require 'drb/drb'
-#   
+#
 #   SERVER_URI="druby://localhost:8787"
 #
 #   DRb.start_service
-#   
+#
 #   log_service=DRbObject.new_with_uri(SERVER_URI)
-#   
+#
 #   ["loga", "logb", "logc"].each do |logname|
-#   
+#
 #       logger=log_service.get_logger(logname)
-#   
+#
 #       logger.log("Hello, world!")
 #       logger.log("Goodbye, world!")
 #       logger.log("=== EOT ===")
-#   
+#
 #   end
 #
 # == Security
@@ -288,9 +288,9 @@ require 'drb/eq'
 #    ro.instance_eval("`rm -rf *`")
 #
 # The dangers posed by instance_eval and friends are such that a
-# DRbServer should generally be run with $SAFE set to at least 
-# level 1.  This will disable eval() and related calls on strings 
-# passed across the wire.  The sample usage code given above follows 
+# DRbServer should generally be run with $SAFE set to at least
+# level 1.  This will disable eval() and related calls on strings
+# passed across the wire.  The sample usage code given above follows
 # this practice.
 #
 # A DRbServer can be configured with an access control list to
@@ -360,7 +360,7 @@ module DRb
   #
   # This, the default implementation, uses an object's local ObjectSpace
   # __id__ as its id.  This means that an object's identification over
-  # drb remains valid only while that object instance remains alive 
+  # drb remains valid only while that object instance remains alive
   # within the server runtime.
   #
   # For alternative mechanisms, see DRb::TimerIdConv in rdb/timeridconv.rb
@@ -374,7 +374,7 @@ module DRb
     def to_obj(ref)
       ObjectSpace._id2ref(ref)
     end
-    
+
     # Convert an object into a reference id.
     #
     # This implementation returns the object's __id__ in the local
@@ -390,7 +390,7 @@ module DRb
   # called over drb, then the object remains in the server space
   # and a reference to the object is returned, rather than the
   # object being marshalled and moved into the client space.
-  module DRbUndumped 
+  module DRbUndumped
     def _dump(dummy)  # :nodoc:
       raise TypeError, 'can\'t dump'
     end
@@ -424,7 +424,7 @@ module DRb
     def self._load(s)  # :nodoc:
       Marshal::load(s)
     end
-    
+
     def _dump(lv) # :nodoc:
       Marshal::dump(@unknown)
     end
@@ -456,11 +456,11 @@ module DRb
   # +name+ attribute.  The marshalled object is held in the +buf+
   # attribute.
   class DRbUnknown
-    
+
     # Create a new DRbUnknown object.
     #
     # +buf+ is a string containing a marshalled object that could not
-    # be unmarshalled.  +err+ is the error message that was raised 
+    # be unmarshalled.  +err+ is the error message that was raised
     # when the unmarshalling failed.  It is used to determine the
     # name of the unmarshalled object.
     def initialize(err, buf)
@@ -499,7 +499,7 @@ module DRb
     # Attempt to load the wrapped marshalled object again.
     #
     # If the class of the object is now known locally, the object
-    # will be unmarshalled and returned.  Otherwise, a new 
+    # will be unmarshalled and returned.  Otherwise, a new
     # but identical DRbUnknown object will be returned.
     def reload
       self.class._load(@buf)
@@ -513,7 +513,7 @@ module DRb
 
   class DRbArray
     def initialize(ary)
-      @ary = ary.collect { |obj| 
+      @ary = ary.collect { |obj|
 	if obj.kind_of? DRbUndumped
 	  DRbObject.new(obj)
 	else
@@ -607,13 +607,13 @@ module DRb
     rescue
       raise(DRbConnError, $!.message, $!.backtrace)
     end
-    
+
     def recv_request(stream) # :nodoc:
       ref = load(stream)
       ro = DRb.to_obj(ref)
       msg = load(stream)
       argc = load(stream)
-      raise ArgumentError, 'too many arguments' if @argc_limit < argc
+      raise(DRbConnError, "too many arguments") if @argc_limit < argc
       argv = Array.new(argc, nil)
       argc.times do |n|
 	argv[n] = load(stream)
@@ -656,10 +656,10 @@ module DRb
   #                              using configuration +config+.  Return a
   #                              protocol instance for this listener.
   #   [uri_option(uri, config)] Take a URI, possibly containing an option
-  #                             component (e.g. a trailing '?param=val'), 
+  #                             component (e.g. a trailing '?param=val'),
   #                             and return a [uri, option] tuple.
   #
-  # All of these methods should raise a DRbBadScheme error if the URI 
+  # All of these methods should raise a DRbBadScheme error if the URI
   # does not identify the protocol they support (e.g. "druby:" for
   # the standard Ruby protocol).  This is how the DRbProtocol module,
   # given a URI, determines which protocol implementation serves that
@@ -675,14 +675,14 @@ module DRb
   #
   # The protocol instance returned by #open must have the following methods:
   #
-  # [send_request (ref, msg_id, arg, b)] 
+  # [send_request (ref, msg_id, arg, b)]
   #      Send a request to +ref+ with the given message id and arguments.
   #      This is most easily implemented by calling DRbMessage.send_request,
   #      providing a stream that sits on top of the current protocol.
   # [recv_reply]
   #      Receive a reply from the server and return it as a [success-boolean,
   #      reply-value] pair.  This is most easily implemented by calling
-  #      DRb.recv_reply, providing a stream that sits on top of the 
+  #      DRb.recv_reply, providing a stream that sits on top of the
   #      current protocol.
   # [alive?]
   #      Is this connection still alive?
@@ -725,7 +725,7 @@ module DRb
     # URI by raising a DRbBadScheme error.  If no protocol recognises the
     # URI, then a DRbBadURI error is raised.  If a protocol accepts the
     # URI, but an error occurs in opening it, a DRbConnError is raised.
-    def open(uri, config, first=true) 
+    def open(uri, config, first=true)
       @protocol.each do |prot|
 	begin
 	  return prot.open(uri, config)
@@ -744,14 +744,14 @@ module DRb
     end
     module_function :open
 
-    # Open a server listening for connections at +uri+ with 
+    # Open a server listening for connections at +uri+ with
     # configuration +config+.
     #
     # The DRbProtocol module asks each registered protocol in turn to
-    # try to open a server at the URI.  Each protocol signals that it does 
-    # not handle that URI by raising a DRbBadScheme error.  If no protocol 
-    # recognises the URI, then a DRbBadURI error is raised.  If a protocol 
-    # accepts the URI, but an error occurs in opening it, the underlying 
+    # try to open a server at the URI.  Each protocol signals that it does
+    # not handle that URI by raising a DRbBadScheme error.  If no protocol
+    # recognises the URI, then a DRbBadURI error is raised.  If a protocol
+    # accepts the URI, but an error occurs in opening it, the underlying
     # error is passed on to the caller.
     def open_server(uri, config, first=true)
       @protocol.each do |prot|
@@ -773,7 +773,7 @@ module DRb
     # The DRbProtocol module asks each registered protocol in turn to
     # try to parse the URI.  Each protocol signals that it does not handle that
     # URI by raising a DRbBadScheme error.  If no protocol recognises the
-    # URI, then a DRbBadURI error is raised.  
+    # URI, then a DRbBadURI error is raised.
     def uri_option(uri, config, first=true)
       @protocol.each do |prot|
 	begin
@@ -837,23 +837,18 @@ module DRb
     end
 
     def self.open_server_inaddr_any(host, port)
-      infos = Socket::getaddrinfo(host, nil, 
+      infos = Socket::getaddrinfo(host, nil,
                                   Socket::AF_UNSPEC,
-                                  Socket::SOCK_STREAM, 
+                                  Socket::SOCK_STREAM,
                                   0,
                                   Socket::AI_PASSIVE)
-      family = infos.collect { |af, *_| af }.uniq
-      case family
-      when ['AF_INET']
-        return TCPServer.open('0.0.0.0', port)
-      when ['AF_INET6']
-        return TCPServer.open('::', port)
-      else
-        return TCPServer.open(port)
-      end
+      families = Hash[*infos.collect { |af, *_| af }.uniq.zip([]).flatten]
+      return TCPServer.open('0.0.0.0', port) if families.has_key?('AF_INET')
+      return TCPServer.open('::', port) if families.has_key?('AF_INET6')
+      return TCPServer.open(port)
     end
 
-    # Open a server listening for connections at +uri+ using 
+    # Open a server listening for connections at +uri+ using
     # configuration +config+.
     def self.open_server(uri, config)
       uri = 'druby://:0' unless uri
@@ -899,7 +894,7 @@ module DRb
     def peeraddr
       @socket.peeraddr
     end
-    
+
     # Get the socket.
     def stream; @socket; end
 
@@ -907,7 +902,7 @@ module DRb
     def send_request(ref, msg_id, arg, b)
       @msg.send_request(stream, ref, msg_id, arg, b)
     end
-    
+
     # On the server side, receive a request from the client.
     def recv_request
       @msg.recv_request(stream)
@@ -937,14 +932,14 @@ module DRb
 	@socket = nil
       end
     end
-    
-    # On the server side, for an instance returned by #open_server, 
+
+    # On the server side, for an instance returned by #open_server,
     # accept a client connection and return a new instance to handle
     # the server's side of this client-server session.
     def accept
       while true
 	s = @socket.accept
-	break if (@acl ? @acl.allow_socket?(s) : true) 
+	break if (@acl ? @acl.allow_socket?(s) : true)
 	s.close
       end
       if @config[:tcp_original_host].to_s.size == 0
@@ -981,16 +976,16 @@ module DRb
     end
     attr :option
     def to_s; @option; end
-    
+
     def ==(other)
       return false unless DRbURIOption === other
       @option == other.option
     end
-    
+
     def hash
       @option.hash
     end
-    
+
     alias eql? ==
   end
 
@@ -1007,7 +1002,7 @@ module DRb
     # created to act as a stub for the remote referenced object.
     def self._load(s)
       uri, ref = Marshal.load(s)
-      
+
       if DRb.here?(uri)
 	obj = DRb.to_obj(ref)
         if ((! obj.tainted?) && Thread.current[:drb_untaint])
@@ -1057,7 +1052,7 @@ module DRb
     end
 
     # Get the URI of the remote object.
-    def __drburi 
+    def __drburi
       @uri
     end
 
@@ -1085,7 +1080,7 @@ module DRb
       if DRb.here?(@uri)
 	obj = DRb.to_obj(@ref)
 	DRb.current_server.check_insecure_method(obj, msg_id)
-	return obj.__send__(msg_id, *a, &b) 
+	return obj.__send__(msg_id, *a, &b)
       end
 
       succ, result = self.class.with_friend(@uri) do
@@ -1108,7 +1103,7 @@ module DRb
     def self.with_friend(uri)
       friend = DRb.fetch_server(uri)
       return yield() unless friend
-      
+
       save = Thread.current['DRb']
       Thread.current['DRb'] = { 'server' => friend }
       return yield
@@ -1120,7 +1115,7 @@ module DRb
       prefix = "(#{uri}) "
       bt = []
       result.backtrace.each do |x|
-        break if /`__send__'$/ =~ x 
+        break if /`__send__'$/ =~ x
         if /^\(druby:\/\// =~ x
           bt.push(x)
         else
@@ -1208,6 +1203,7 @@ module DRb
     end
 
     def alive?  # :nodoc:
+      return false unless @protocol
       @protocol.alive?
     end
   end
@@ -1270,14 +1266,14 @@ module DRb
     def self.verbose=(on)
       @@verbose = on
     end
-    
+
     # Get the default value of the :verbose option.
     def self.verbose
       @@verbose
     end
 
     def self.make_config(hash={})  # :nodoc:
-      default_config = { 
+      default_config = {
 	:idconv => @@idconv,
 	:verbose => @@verbose,
 	:tcp_acl => @@acl,
@@ -1367,7 +1363,7 @@ module DRb
     attr_reader :thread
 
     # The front object of the DRbServer.
-    # 
+    #
     # This object receives remote method calls made on the server's
     # URI alone, with an object id.
     attr_reader :front
@@ -1398,7 +1394,7 @@ module DRb
       if  Thread.current['DRb'] && Thread.current['DRb']['server'] == self
         Thread.current['DRb']['stop_service'] = true
       else
-        @thread.kill
+        @thread.kill.join
       end
     end
 
@@ -1460,7 +1456,7 @@ module DRb
     def any_to_s(obj)
       obj.to_s + ":#{obj.class}"
     rescue
-      sprintf("#<%s:0x%lx>", obj.class, obj.__id__)      
+      sprintf("#<%s:0x%lx>", obj.class, obj.__id__)
     end
 
     # Check that a method is callable via dRuby.
@@ -1468,14 +1464,14 @@ module DRb
     # +obj+ is the object we want to invoke the method on. +msg_id+ is the
     # method name, as a Symbol.
     #
-    # If the method is an insecure method (see #insecure_method?) a 
+    # If the method is an insecure method (see #insecure_method?) a
     # SecurityError is thrown.  If the method is private or undefined,
     # a NameError is thrown.
     def check_insecure_method(obj, msg_id)
       return true if Proc === obj && msg_id == :__drb_yield
       raise(ArgumentError, "#{any_to_s(msg_id)} is not a symbol") unless Symbol == msg_id.class
       raise(SecurityError, "insecure method `#{msg_id}'") if insecure_method?(msg_id)
-      
+
       if obj.private_methods.include?(msg_id)
 	desc = any_to_s(obj)
         raise NoMethodError, "private method `#{msg_id}' called for #{desc}"
@@ -1487,7 +1483,7 @@ module DRb
       end
     end
     public :check_insecure_method
-    
+
     class InvokeMethod  # :nodoc:
       def initialize(drb_server, client)
 	@drb_server = drb_server
@@ -1509,7 +1505,7 @@ module DRb
               perform_with_block
             }.value
           else
-            @result = Thread.new { 
+            @result = Thread.new {
               Thread.current['DRb'] = info
               $SAFE = @safe_level
               perform_without_block
@@ -1524,7 +1520,7 @@ module DRb
         end
 	@succ = true
 	if @msg_id == :to_ary && @result.class == Array
-	  @result = DRbArray.new(@result) 
+	  @result = DRbArray.new(@result)
 	end
 	return @succ, @result
       rescue StandardError, ScriptError, Interrupt
@@ -1540,7 +1536,7 @@ module DRb
         @argv = argv
         @block = block
       end
-      
+
       def check_insecure_method
         @drb_server.check_insecure_method(@obj, @msg_id)
       end
@@ -1549,7 +1545,7 @@ module DRb
 	init_with_client
 	check_insecure_method
       end
-      
+
       def perform_without_block
 	if Proc === @obj && @msg_id == :__drb_yield
           if @argv.size == 1
@@ -1637,7 +1633,7 @@ module DRb
 
   # The primary local dRuby server.
   #
-  # This is the server created by the #start_service call.  
+  # This is the server created by the #start_service call.
   attr_accessor :primary_server
   module_function :primary_server=, :primary_server
 
@@ -1652,8 +1648,8 @@ module DRb
   # If the above rule fails to find a server, a DRbServerNotFound
   # error is raised.
   def current_server
-    drb = Thread.current['DRb'] 
-    server = (drb && drb['server']) ? drb['server'] : @primary_server 
+    drb = Thread.current['DRb']
+    server = (drb && drb['server']) ? drb['server'] : @primary_server
     raise DRbServerNotFound unless server
     return server
   end
@@ -1699,7 +1695,7 @@ module DRb
     DRbServer.make_config
   end
   module_function :config
-  
+
   # Get the front object of the current server.
   #
   # This raises a DRbServerNotFound error if there is no current server.
@@ -1770,7 +1766,7 @@ module DRb
     @server.delete(server.uri)
   end
   module_function :remove_server
-  
+
   def fetch_server(uri)
     @server[uri]
   end
