@@ -1,13 +1,12 @@
 #
-#   irb/ruby-lex.rb - ruby lexcal analizer
-#   	$Release Version: 0.9.5$
-#   	$Revision: 12889 $
-#   	$Date: 2007-08-07 00:11:33 +0900 (Tue, 07 Aug 2007) $
+#   irb/ruby-lex.rb - ruby lexcal analyzer
+#   	$Release Version: 0.9.6$
+#   	$Revision: 32916 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
-#   
+#
 #
 
 require "e2mmap"
@@ -15,18 +14,18 @@ require "irb/slex"
 require "irb/ruby-token"
 
 class RubyLex
-  @RCS_ID='-$Id: ruby-lex.rb 12889 2007-08-06 15:11:33Z keiju $-'
+  @RCS_ID='-$Id: ruby-lex.rb 32916 2011-08-11 00:38:19Z yugui $-'
 
   extend Exception2MessageMapper
   def_exception(:AlreadyDefinedToken, "Already defined token(%s)")
   def_exception(:TkReading2TokenNoKey, "key nothing(key='%s')")
   def_exception(:TkSymbol2TokenNoKey, "key nothing(key='%s')")
-  def_exception(:TkReading2TokenDuplicateError, 
+  def_exception(:TkReading2TokenDuplicateError,
 		"key duplicate(token_n='%s', key='%s')")
   def_exception(:SyntaxError, "%s")
 
   def_exception(:TerminateLineInput, "Terminate Line Input")
-  
+
   include RubyToken
 
   class << self
@@ -54,7 +53,7 @@ class RubyLex
     @lex_state = EXPR_BEG
     @space_seen = false
     @here_header = false
-    
+
     @continue = false
     @line = ""
 
@@ -92,7 +91,7 @@ class RubyLex
     else
       @base_char_no += @readed.size
     end
-    
+
     readed = @readed.join("")
     @readed = []
     readed
@@ -104,7 +103,6 @@ class RubyLex
       @rests.push nil unless buf_input
     end
     c = @rests.shift
-    return if c == nil
     if @here_header
       @here_readed.push c
     else
@@ -112,7 +110,7 @@ class RubyLex
     end
     @seek += 1
     if c == "\n"
-      @line_no += 1 
+      @line_no += 1
       @char_no = 0
     else
       @char_no += 1
@@ -149,10 +147,10 @@ class RubyLex
       c2 = @here_readed.pop
     end
     c = c2 unless c
-    @rests.unshift c #c = 
+    @rests.unshift c #c =
       @seek -= 1
     if c == "\n"
-      @line_no -= 1 
+      @line_no -= 1
       if idx = @readed.reverse.index("\n")
 	@char_no = @readed.size - idx
       else
@@ -189,7 +187,7 @@ class RubyLex
     prompt
     line = @input.call
     return nil unless line
-    @rests.concat line.split(//)
+    @rests.concat line.chars.to_a
     true
   end
   private :buf_input
@@ -217,14 +215,14 @@ class RubyLex
     @lex_state = EXPR_BEG
     @space_seen = false
     @here_header = false
-    
+
     @continue = false
     prompt
 
     @line = ""
     @exp_line_no = @line_no
   end
-  
+
   def each_top_level_statement
     initialize_input
     catch(:TERM_INPUT) do
@@ -241,6 +239,7 @@ class RubyLex
 	    end
 	  end
 	  if @line != "\n"
+      @line.force_encoding(@io.encoding)
 	    yield @line, @exp_line_no
 	  end
 	  break unless l
@@ -297,7 +296,7 @@ class RubyLex
     #      Tracer.off
     tk
   end
-  
+
   ENINDENT_CLAUSE = [
     "case", "class", "def", "do", "for", "if",
     "module", "unless", "until", "while", "begin" #, "when"
@@ -314,7 +313,7 @@ class RubyLex
     "W" => "]",
     "s" => ":"
   }
-  
+
   PERCENT_PAREN = {
     "{" => "}",
     "[" => "]",
@@ -354,7 +353,7 @@ class RubyLex
     end
 
     @OP.def_rule("=begin",
-		 proc{|op, io| @prev_char_no == 0 && peek(0) =~ /\s/}) do 
+		 proc{|op, io| @prev_char_no == 0 && peek(0) =~ /\s/}) do
       |op, io|
       @ltype = "="
       until getc == "\n"; end
@@ -374,8 +373,8 @@ class RubyLex
       else
 	@continue = false
 	@lex_state = EXPR_BEG
-	until (@indent_stack.empty? || 
-	       [TkLPAREN, TkLBRACK, TkLBRACE, 
+	until (@indent_stack.empty? ||
+	       [TkLPAREN, TkLBRACK, TkLBRACE,
 		 TkfLPAREN, TkfLBRACK, TkfLBRACE].include?(@indent_stack.last))
 	  @indent_stack.pop
 	end
@@ -385,11 +384,12 @@ class RubyLex
       Token(TkNL)
     end
 
-    @OP.def_rules("*", "**",	
-		  "=", "==", "===", 
-		  "=~", "<=>",	
+    @OP.def_rules("*", "**",
+		  "=", "==", "===",
+		  "=~", "<=>",
 		  "<", "<=",
-		  ">", ">=", ">>") do
+		  ">", ">=", ">>",
+		  "!", "!=", "!~") do
       |op, io|
       case @lex_state
       when EXPR_FNAME, EXPR_DOT
@@ -400,19 +400,13 @@ class RubyLex
       Token(op)
     end
 
-    @OP.def_rules("!", "!=", "!~") do
-      |op, io|
-      @lex_state = EXPR_BEG
-      Token(op)
-    end
-
     @OP.def_rules("<<") do
       |op, io|
       tk = nil
       if @lex_state != EXPR_END && @lex_state != EXPR_CLASS &&
 	  (@lex_state != EXPR_ARG || @space_seen)
 	c = peek(0)
-	if /\S/ =~ c && (/["'`]/ =~ c || /[\w_]/ =~ c || c == "-")
+	if /\S/ =~ c && (/["'`]/ =~ c || /\w/ =~ c || c == "-")
 	  tk = identify_here_document
 	end
       end
@@ -455,7 +449,7 @@ class RubyLex
 	  @lex_state = EXPR_BEG;
 	  Token(TkQUESTION)
 	else
-	  if (ch == '\\') 
+	  if (ch == '\\')
 	    read_escape
 	  end
 	  @lex_state = EXPR_END
@@ -469,8 +463,8 @@ class RubyLex
       @lex_state = EXPR_BEG
       Token(op)
     end
-    
-    @OP.def_rules("+=", "-=", "*=", "**=", 
+
+    @OP.def_rules("+=", "-=", "*=", "**=",
 		  "&=", "|=", "^=", "<<=", ">>=", "||=", "&&=") do
       |op, io|
       @lex_state = EXPR_BEG
@@ -529,7 +523,7 @@ class RubyLex
 
     lex_int2
   end
-  
+
   def lex_int2
     @OP.def_rules("]", "}", ")") do
       |op, io|
@@ -572,7 +566,7 @@ class RubyLex
 	Token(TkOPASGN, "/") #/)
       elsif @lex_state == EXPR_ARG and @space_seen and peek(0) !~ /\s/
 	identify_string(op)
-      else 
+      else
 	@lex_state = EXPR_BEG
 	Token("/") #/)
       end
@@ -588,7 +582,7 @@ class RubyLex
     # 	@lex_state = EXPR_BEG
     # 	Token(OP_ASGN, :^)
     #       end
-    
+
     @OP.def_rules(",") do
       |op, io|
       @lex_state = EXPR_BEG
@@ -598,8 +592,8 @@ class RubyLex
     @OP.def_rules(";") do
       |op, io|
       @lex_state = EXPR_BEG
-      until (@indent_stack.empty? || 
-	     [TkLPAREN, TkLBRACK, TkLBRACE, 
+      until (@indent_stack.empty? ||
+	     [TkLPAREN, TkLBRACK, TkLBRACE,
 	       TkfLPAREN, TkfLBRACK, TkfLBRACE].include?(@indent_stack.last))
 	@indent_stack.pop
       end
@@ -617,7 +611,7 @@ class RubyLex
       @lex_state = EXPR_BEG
       Token("~")
     end
-    
+
     @OP.def_rule("(") do
       |op, io|
       @indent += 1
@@ -683,7 +677,7 @@ class RubyLex
 	@continue = true
 	Token(TkSPACE)
       else
-	ungetc
+	read_escape
 	Token("\\")
       end
     end
@@ -710,7 +704,7 @@ class RubyLex
 
     @OP.def_rule('@') do
       |op, io|
-      if peek(0) =~ /[\w_@]/
+      if peek(0) =~ /[\w@]/
 	ungetc
 	identify_identifier
       else
@@ -718,7 +712,7 @@ class RubyLex
       end
     end
 
-    #       @OP.def_rule("def", proc{|op, io| /\s/ =~ io.peek(0)}) do 
+    #       @OP.def_rule("def", proc{|op, io| /\s/ =~ io.peek(0)}) do
     # 	|op, io|
     # 	@indent += 1
     # 	@lex_state = EXPR_FNAME
@@ -733,19 +727,19 @@ class RubyLex
       printf "MATCH: start %s: %s\n", op, io.inspect if RubyLex.debug?
       if peek(0) =~ /[0-9]/
 	t = identify_number
-      elsif peek(0) =~ /[\w_]/
+      elsif peek(0) =~ /[^\x00-\/:-@\[-^`{-\x7F]/
 	t = identify_identifier
       end
       printf "MATCH: end %s: %s\n", op, io.inspect if RubyLex.debug?
       t
     end
-    
+
     p @OP if RubyLex.debug?
   end
-  
+
   def identify_gvar
     @lex_state = EXPR_END
-    
+
     case ch = getc
     when /[~_*$?!@\/\\;,=:<>".]/   #"
       Token(TkGVAR, "$" + ch)
@@ -761,12 +755,12 @@ class RubyLex
       ungetc
       ungetc
       identify_identifier
-    else 
+    else
       ungetc
       Token("$")
     end
   end
-  
+
   def identify_identifier
     token = ""
     if peek(0) =~ /[$@]/
@@ -776,12 +770,12 @@ class RubyLex
       end
     end
 
-    while (ch = getc) =~ /\w|_/
+    while (ch = getc) =~ /[^\x00-\/:-@\[-^`{-\x7F]/
       print ":", ch, ":" if RubyLex.debug?
       token.concat ch
     end
     ungetc
-    
+
     if (ch == "!" || ch == "?") && token[0,1] =~ /\w/ && peek(0) != "="
       token.concat getc
     end
@@ -799,7 +793,7 @@ class RubyLex
       @lex_state = EXPR_END
       return Token(TkIVAR, token)
     end
-    
+
     if @lex_state != EXPR_DOT
       print token, "\n" if RubyLex.debug?
 
@@ -822,11 +816,11 @@ class RubyLex
 	      when "class"
 		valid = false unless peek_match?(/^\s*(<<|\w|::)/)
 	      when "def"
-		valid = false if peek_match?(/^\s*(([+-\/*&\|^]|<<|>>|\|\||\&\&)=|\&\&|\|\|)/)
+		valid = false if peek_match?(/^\s*(([+\-\/*&\|^]|<<|>>|\|\||\&\&)=|\&\&|\|\|)/)
 	      when "do"
-		valid = false if peek_match?(/^\s*([+-\/*]?=|\*|<|>|\&)/)
+		valid = false if peek_match?(/^\s*([+\-\/*]?=|\*|<|>|\&)/)
 	      when *ENINDENT_CLAUSE
-		valid = false if peek_match?(/^\s*([+-\/*]?=|\*|<|>|\&|\|)/)
+		valid = false if peek_match?(/^\s*([+\-\/*]?=|\*|<|>|\&|\|)/)
 	      else
 		# no nothing
 	      end
@@ -927,7 +921,7 @@ class RubyLex
     @lex_state = EXPR_END
     Token(Ltype2Token[lt])
   end
-  
+
   def identify_quotation
     ch = getc
     if lt = PERCENT_LTYPE[ch]
@@ -967,11 +961,11 @@ class RubyLex
       when /[0-7]/
 	match = /[0-7_]/
       when /[89]/
-	RubyLex.fail SyntaxError, "Illegal octal digit"
-      else 
+	RubyLex.fail SyntaxError, "Invalid octal digit"
+      else
 	return Token(TkINTEGER)
       end
-      
+
       len0 = true
       non_digit = false
       while ch = getc
@@ -999,7 +993,7 @@ class RubyLex
       end
       return Token(TkINTEGER)
     end
-    
+
     type = TkINTEGER
     allow_point = true
     allow_e = true
@@ -1042,7 +1036,7 @@ class RubyLex
     end
     Token(type)
   end
-  
+
   def identify_string(ltype, quoted = ltype)
     @ltype = ltype
     @quoted = quoted
@@ -1052,6 +1046,8 @@ class RubyLex
       while ch = getc
 	if @quoted == ch and nest == 0
 	  break
+	elsif @ltype != "'" && ch == "#" && peek(0) == "{"
+	  identify_string_dvar
 	elsif @ltype != "'" && @ltype != "]" && @ltype != ":" and ch == "#"
 	  subtype = true
 	elsif ch == '\\' and @ltype == "'" #'
@@ -1063,7 +1059,7 @@ class RubyLex
 	elsif ch == '\\' #'
 	  read_escape
 	end
-	if PERCENT_PAREN.values.include?(@quoted) 
+	if PERCENT_PAREN.values.include?(@quoted)
 	  if PERCENT_PAREN[ch] == @quoted
 	    nest += 1
 	  elsif ch == @quoted
@@ -1072,7 +1068,7 @@ class RubyLex
 	end
       end
       if @ltype == "/"
-	if peek(0) =~ /i|m|x|o|e|s|u|n/
+       while peek(0) =~ /i|m|x|o|e|s|u|n/
 	  getc
 	end
       end
@@ -1087,6 +1083,42 @@ class RubyLex
       @lex_state = EXPR_END
     end
   end
+
+  def identify_string_dvar
+    begin
+      getc
+
+      reserve_continue = @continue
+      reserve_ltype = @ltype
+      reserve_indent = @indent
+      reserve_indent_stack = @indent_stack
+      reserve_state = @lex_state
+      reserve_quoted = @quoted
+
+      @ltype = nil
+      @quoted = nil
+      @indent = 0
+      @indent_stack = []
+      @lex_state = EXPR_BEG
+      
+      loop do
+	@continue = false
+	prompt
+	tk = token
+	if @ltype or @continue or @indent > 0
+	  next
+	end
+	break if tk.kind_of?(TkRBRACE)
+      end
+    ensure
+      @continue = reserve_continue
+      @ltype = reserve_ltype
+      @indent = reserve_indent
+      @indent_stack = reserve_indent_stack
+      @lex_state = reserve_state
+      @quoted = reserve_quoted
+    end
+  end
   
   def identify_comment
     @ltype = "#"
@@ -1103,7 +1135,7 @@ class RubyLex
     end
     return Token(TkCOMMENT)
   end
-  
+
   def read_escape
     case ch = getc
     when "\n", "\r", "\f"
@@ -1120,7 +1152,7 @@ class RubyLex
 	  break
 	end
       end
-      
+
     when "x"
       2.times do
 	case ch = getc
@@ -1149,7 +1181,7 @@ class RubyLex
 	read_escape
       end
     else
-      # other characters 
+      # other characters
     end
   end
 end
