@@ -1,13 +1,12 @@
 #
-#   irb/workspace-binding.rb - 
-#   	$Release Version: 0.9.5$
-#   	$Revision: 11708 $
-#   	$Date: 2007-02-13 08:01:19 +0900 (火, 13  2月 2007) $
+#   irb/workspace-binding.rb -
+#   	$Release Version: 0.9.6$
+#   	$Revision: 25189 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
-#   
+#
 #
 module IRB
   class WorkSpace
@@ -22,7 +21,7 @@ module IRB
 	case IRB.conf[:CONTEXT_MODE]
 	when 0	# binding in proc on TOPLEVEL_BINDING
 	  @binding = eval("proc{binding}.call",
-		      TOPLEVEL_BINDING, 
+		      TOPLEVEL_BINDING,
 		      __FILE__,
 		      __LINE__)
 	when 1	# binding in loaded file
@@ -38,7 +37,7 @@ EOF
 	when 2	# binding in loaded file(thread use)
 	  unless defined? BINDING_QUEUE
 	    require "thread"
-	    
+
 	    IRB.const_set("BINDING_QUEUE", SizedQueue.new(1))
 	    Thread.abort_on_exception = true
 	    Thread.start do
@@ -50,7 +49,7 @@ EOF
 
 	when 3	# binging in function on TOPLEVEL_BINDING(default)
 	  @binding = eval("def irb_binding; binding; end; irb_binding",
-		      TOPLEVEL_BINDING, 
+		      TOPLEVEL_BINDING,
 		      __FILE__,
 		      __LINE__ - 3)
 	end
@@ -64,7 +63,7 @@ EOF
 	when Module
 	  @binding = eval("IRB.conf[:__MAIN__].module_eval('binding', __FILE__, __LINE__)", @binding, __FILE__, __LINE__)
 	else
-	  begin 
+	  begin
 	    @binding = eval("IRB.conf[:__MAIN__].instance_eval('binding', __FILE__, __LINE__)", @binding, __FILE__, __LINE__)
 	  rescue TypeError
 	    IRB.fail CantChangeBinding, @main.inspect
@@ -80,7 +79,7 @@ EOF
     def evaluate(context, statements, file = __FILE__, line = __LINE__)
       eval(statements, @binding, file, line)
     end
-  
+
     # error message manipulator
     def filter_backtrace(bt)
       case IRB.conf[:CONTEXT_MODE]
@@ -94,9 +93,11 @@ EOF
 	end
       when 2
 	return nil if bt =~ /irb\/.*\.rb/
+	return nil if bt =~ /irb\.rb/
       when 3
 	return nil if bt =~ /irb\/.*\.rb/
-	bt.sub!(/:\s*in `irb_binding'/){""} 
+	return nil if bt =~ /irb\.rb/
+	bt.sub!(/:\s*in `irb_binding'/, '')
       end
       bt
     end
