@@ -32,9 +32,6 @@ when /darwin/
 when /kfreebsd/
   libc_so = "/lib/libc.so.0.1"
   libm_so = "/lib/libm.so.1"
-when /gnu/	#GNU/Hurd
-  libc_so = "/lib/libc.so.0.3"
-  libm_so = "/lib/libm.so.6"
 when /bsd|dragonfly/
   libc_so = "/usr/lib/libc.so"
   libm_so = "/usr/lib/libm.so"
@@ -48,8 +45,8 @@ when /solaris/
     # 64-bit ruby
     libdir = '/lib/64' if File.directory? '/lib/64'
   end
-  libc_so = File.join(libdir, "libc.so")
-  libm_so = File.join(libdir, "libm.so")
+  libc_so = File.join(libdir, "libc.so.6")
+  libm_so = File.join(libdir, "libm.so.6")
 when /aix/
   pwd=Dir.pwd
   libc_so = libm_so = "#{pwd}/libaixdltest.so"
