@@ -1,4 +1,4 @@
-require 'psych/helper'
+require 'psych/handler'
 
 module Psych
   class TestStringTainted < TestCase
@@ -121,9 +121,7 @@ module Psych
       t.binmode
       t.write string
       t.close
-      File.open(t.path, 'r:bom|utf-8') { |f|
-        @parser.parse f
-      }
+      File.open(t.path) { |f| @parser.parse f }
       t.close(true)
     end
   end
