@@ -1,8 +1,8 @@
 require 'test/unit'
 
 class TestEnv < Test::Unit::TestCase
-  IGNORE_CASE = /djgpp|bccwin|mswin|mingw/ =~ RUBY_PLATFORM
-  PATH_ENV = /human68k/ =~ RUBY_PLATFORM ? "path" : "PATH"
+  IGNORE_CASE = /bccwin|mswin|mingw/ =~ RUBY_PLATFORM
+  PATH_ENV = "PATH"
 
   def setup
     @verbose = $VERBOSE
@@ -49,7 +49,7 @@ class TestEnv < Test::Unit::TestCase
 
   def test_has_value
     val = 'a'
-    val.succ! while ENV.has_value?(val) && ENV.has_value?(val.upcase)
+    val.succ! while ENV.has_value?(val) || ENV.has_value?(val.upcase)
     ENV['test'] = val[0...-1]
 
     assert_equal(false, ENV.has_value?(val))
@@ -64,10 +64,11 @@ class TestEnv < Test::Unit::TestCase
 
   def test_key
     val = 'a'
-    val.succ! while ENV.has_value?(val) && ENV.has_value?(val.upcase)
+    val.succ! while ENV.has_value?(val) || ENV.has_value?(val.upcase)
     ENV['test'] = val[0...-1]
 
     assert_nil(ENV.key(val))
+    assert_nil(ENV.index(val))
     assert_nil(ENV.key(val.upcase))
     ENV['test'] = val
     if IGNORE_CASE
@@ -95,6 +96,7 @@ class TestEnv < Test::Unit::TestCase
     assert_raise(ArgumentError) { ENV["foo\0bar"] }
     ENV[PATH_ENV] = ""
     assert_equal("", ENV[PATH_ENV])
+    assert_nil(ENV[""])
   end
 
   def test_fetch
@@ -122,6 +124,13 @@ class TestEnv < Test::Unit::TestCase
     assert_equal(nil, ENV["test"])
     assert_raise(ArgumentError) { ENV["foo\0bar"] = "test" }
     assert_raise(ArgumentError) { ENV["test"] = "foo\0bar" }
+    if /netbsd|openbsd/ =~ RUBY_PLATFORM
+      ENV["foo=bar"] = "test"
+      assert_equal("test", ENV["foo=bar"])
+      assert_equal("test", ENV["foo"])
+    else
+      assert_raise(Errno::EINVAL) { ENV["foo=bar"] = "test" }
+    end
     ENV[PATH_ENV] = "/tmp/".taint
     assert_equal("/tmp/", ENV[PATH_ENV])
   end
@@ -173,6 +182,24 @@ class TestEnv < Test::Unit::TestCase
     assert_equal(h1, h2)
   end
 
+  def test_select_bang
+    h1 = {}
+    ENV.each_pair {|k, v| h1[k] = v }
+    ENV["test"] = "foo"
+    ENV.select! {|k, v| IGNORE_CASE ? k.upcase != "TEST" : k != "test" }
+    h2 = {}
+    ENV.each_pair {|k, v| h2[k] = v }
+    assert_equal(h1, h2)
+
+    h1 = {}
+    ENV.each_pair {|k, v| h1[k] = v }
+    ENV["test"] = "foo"
+    ENV.keep_if {|k, v| IGNORE_CASE ? k.upcase != "TEST" : k != "test" }
+    h2 = {}
+    ENV.each_pair {|k, v| h2[k] = v }
+    assert_equal(h1, h2)
+  end
+
   def test_values_at
     ENV["test"] = "foo"
     assert_equal(["foo", "foo"], ENV.values_at("test", "test"))
@@ -348,4 +375,18 @@ class TestEnv < Test::Unit::TestCase
     ENV.update({"baz"=>"quux","a"=>"b"}) {|k, v1, v2| v1 ? k + "_" + v1 + "_" + v2 : v2 }
     check(ENV.to_hash.to_a, [%w(foo bar), %w(baz baz_qux_quux), %w(a b)])
   end
+
+  def test_huge_value
+    huge_value = "bar" * 40960
+    ENV["foo"] = "bar"
+    if /mswin|mingw/ =~ RUBY_PLATFORM
+      warning = verbose_warning { ENV["foo"] = huge_value }
+      assert_match(/failed to set environment variable/, warning)
+      assert_match(/Ruby 1\.9\.3/, warning)
+      assert_equal("bar", ENV["foo"])
+    else
+      assert_nothing_raised { ENV["foo"] = huge_value }
+      assert_equal(huge_value, ENV["foo"])
+    end
+  end
 end
