@@ -1,12 +1,4 @@
-#!/usr/bin/env ruby
-#--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
-# All rights reserved.
-# See LICENSE.txt for permissions.
-#++
-
-require 'test/unit'
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require_relative 'gemutilities'
 require 'rubygems/config_file'
 
 class TestGemConfigFile < RubyGemTestCase
@@ -54,6 +46,7 @@ class TestGemConfigFile < RubyGemTestCase
       fp.puts ":benchmark: true"
       fp.puts ":bulk_threshold: 10"
       fp.puts ":verbose: false"
+      fp.puts ":rubygems_api_key: 701229f217cdf23b1344c7b4b54ca97"
       fp.puts ":sources:"
       fp.puts "  - http://more-gems.example.com"
       fp.puts "install: --wrappers"
@@ -69,6 +62,7 @@ class TestGemConfigFile < RubyGemTestCase
     assert_equal 10, @cfg.bulk_threshold
     assert_equal false, @cfg.verbose
     assert_equal false, @cfg.update_sources
+    assert_equal "701229f217cdf23b1344c7b4b54ca97", @cfg.rubygems_api_key
     assert_equal %w[http://more-gems.example.com], Gem.sources
     assert_equal '--wrappers', @cfg[:install]
     assert_equal(['/usr/ruby/1.8/lib/ruby/gems/1.8', '/var/ruby/1.8/gem_home'],
@@ -273,6 +267,18 @@ class TestGemConfigFile < RubyGemTestCase
     assert_equal %w[http://even-more-gems.example.com], Gem.sources
   end
 
+  def test_load_rubygems_api_key_from_credentials
+    temp_cred = File.join Gem.user_home, '.gem', 'credentials'
+    FileUtils.mkdir File.dirname(temp_cred)
+    File.open temp_cred, 'w' do |fp|
+      fp.puts ":rubygems_api_key: 701229f217cdf23b1344c7b4b54ca97"
+    end
+
+    util_config_file
+
+    assert_equal "701229f217cdf23b1344c7b4b54ca97", @cfg.rubygems_api_key
+  end
+
   def util_config_file(args = @cfg_args)
     @cfg = Gem::ConfigFile.new args
   end
