@@ -1,7 +1,7 @@
-require 'rubygems/test_case'
+require_relative 'gemutilities'
 require 'rubygems/spec_fetcher'
 
-class TestGemSpecFetcher < Gem::TestCase
+class TestGemSpecFetcher < RubyGemTestCase
 
   def setup
     super
@@ -10,43 +10,43 @@ class TestGemSpecFetcher < Gem::TestCase
 
     util_setup_fake_fetcher
 
-    @a_pre = new_spec 'a', '1.a'
+    @a_pre = quick_gem 'a', '1.a'
+    @source_index.add_spec @pl1
+    @source_index.add_spec @a_pre
 
-    install_specs @a_pre
-
-    Gem::Specification.remove_spec @b2
-
-    @specs = Gem::Specification.map { |spec|
+    @specs = @source_index.gems.sort.map do |name, spec|
       [spec.name, spec.version, spec.original_platform]
-    }.sort
+    end.sort
 
-    # TODO: couldn't all of this come from the fake spec fetcher?
-    @latest_specs = Gem::Specification.latest_specs.sort.map { |spec|
+    @latest_specs = @source_index.latest_specs.sort.map do |spec|
       [spec.name, spec.version, spec.original_platform]
-    }
+    end
 
-    prerelease = Gem::Specification.find_all { |s| s.version.prerelease? }
-    @prerelease_specs = prerelease.map { |spec|
+    @prerelease_specs = @source_index.prerelease_gems.sort.map do |name, spec|
       [spec.name, spec.version, spec.original_platform]
-    }.sort
+    end.sort
 
-    v = Gem.marshal_version
-    s_zip = util_gzip(Marshal.dump(@specs))
-    l_zip = util_gzip(Marshal.dump(@latest_specs))
-    p_zip = util_gzip(Marshal.dump(@prerelease_specs))
-    @fetcher.data["#{@gem_repo}specs.#{v}.gz"]            = s_zip
-    @fetcher.data["#{@gem_repo}latest_specs.#{v}.gz"]     = l_zip
-    @fetcher.data["#{@gem_repo}prerelease_specs.#{v}.gz"] = p_zip
+    @fetcher.data["#{@gem_repo}specs.#{Gem.marshal_version}.gz"] =
+      util_gzip(Marshal.dump(@specs))
+
+    @fetcher.data["#{@gem_repo}latest_specs.#{Gem.marshal_version}.gz"] =
+      util_gzip(Marshal.dump(@latest_specs))
+
+    @fetcher.data["#{@gem_repo}prerelease_specs.#{Gem.marshal_version}.gz"] =
+      util_gzip(Marshal.dump(@prerelease_specs))
 
     @sf = Gem::SpecFetcher.new
   end
 
   def test_fetch_all
-    d = "#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}"
-    @fetcher.data["#{d}#{@a1.spec_name}.rz"]    = util_zip(Marshal.dump(@a1))
-    @fetcher.data["#{d}#{@a2.spec_name}.rz"]    = util_zip(Marshal.dump(@a2))
-    @fetcher.data["#{d}#{@a_pre.spec_name}.rz"] = util_zip(Marshal.dump(@a_pre))
-    @fetcher.data["#{d}#{@a3a.spec_name}.rz"]   = util_zip(Marshal.dump(@a3a))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a1.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a1))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a2.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a2))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a_pre.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a_pre))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a3a.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a3a))
 
     dep = Gem::Dependency.new 'a', 1
 
@@ -64,10 +64,12 @@ class TestGemSpecFetcher < Gem::TestCase
   end
 
   def test_fetch_latest
-    d = "#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}"
-    @fetcher.data["#{d}#{@a1.spec_name}.rz"]    = util_zip(Marshal.dump(@a1))
-    @fetcher.data["#{d}#{@a2.spec_name}.rz"]    = util_zip(Marshal.dump(@a2))
-    @fetcher.data["#{d}#{@a_pre.spec_name}.rz"] = util_zip(Marshal.dump(@a_pre))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a1.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a1))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a2.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a2))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a_pre.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a_pre))
 
     dep = Gem::Dependency.new 'a', 1
     specs_and_sources = @sf.fetch dep
@@ -80,12 +82,15 @@ class TestGemSpecFetcher < Gem::TestCase
   end
 
   def test_fetch_prerelease
-    d = "#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}"
-    @fetcher.data["#{d}#{@a1.spec_name}.rz"]    = util_zip(Marshal.dump(@a1))
-    @fetcher.data["#{d}#{@a2.spec_name}.rz"]    = util_zip(Marshal.dump(@a2))
-    @fetcher.data["#{d}#{@a_pre.spec_name}.rz"] = util_zip(Marshal.dump(@a_pre))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a1.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a1))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a2.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a2))
+    @fetcher.data["#{@gem_repo}#{Gem::MARSHAL_SPEC_DIR}#{@a_pre.spec_name}.rz"] =
+      util_zip(Marshal.dump(@a_pre))
 
-    specs_and_sources = @sf.fetch dep('a', '1.a'), false, true, true
+    dep = Gem::Dependency.new 'a', '1.a'
+    specs_and_sources = @sf.fetch dep, false, true, true
 
     spec_names = specs_and_sources.map do |spec, source_uri|
       [spec.full_name, source_uri]
@@ -282,11 +287,11 @@ class TestGemSpecFetcher < Gem::TestCase
 
     assert_equal [@uri], specs.keys
 
-    assert_equal([["a",      Gem::Version.new("1"),   "ruby"],
-                  ["a",      Gem::Version.new("2"),   "ruby"],
-                  ["a_evil", Gem::Version.new("9"),   "ruby"],
-                  ["c",      Gem::Version.new("1.2"), "ruby"],
-                  ["pl",     Gem::Version.new("1"),   "i386-linux"]],
+    assert_equal([["a", Gem::Version.new("1"), "ruby"],
+                  ["a", Gem::Version.new("2"), "ruby"],
+                  ["a_evil", Gem::Version.new("9"), "ruby"],
+                  ["c", Gem::Version.new("1.2"), "ruby"],
+                  ["pl", Gem::Version.new("1"), "i386-linux"]],
                  specs[@uri].sort)
   end
 
@@ -336,17 +341,19 @@ class TestGemSpecFetcher < Gem::TestCase
   end
 
   def test_load_specs
+    specs = @sf.load_specs @uri, 'specs'
+
     expected = [
-      ['a',      Gem::Version.new('1.a'), Gem::Platform::RUBY],
+      ['a',      Gem::Version.new('1.a'),     Gem::Platform::RUBY],
       ['a',      Gem::Version.new(1),     Gem::Platform::RUBY],
       ['a',      Gem::Version.new(2),     Gem::Platform::RUBY],
-      ['a',      Gem::Version.new('3.a'), Gem::Platform::RUBY],
+      ['a',      Gem::Version.new('3.a'),     Gem::Platform::RUBY],
       ['a_evil', Gem::Version.new(9),     Gem::Platform::RUBY],
       ['c',      Gem::Version.new('1.2'), Gem::Platform::RUBY],
       ['pl',     Gem::Version.new(1),     'i386-linux'],
     ]
 
-    assert_equal expected, @sf.load_specs(@uri, 'specs')
+    assert_equal expected, specs
 
     cache_dir = File.join Gem.user_home, '.gem', 'specs', 'gems.example.com%80'
     assert File.exist?(cache_dir), "#{cache_dir} does not exist"
@@ -399,10 +406,5 @@ class TestGemSpecFetcher < Gem::TestCase
     assert_equal @latest_specs, latest_specs
   end
 
-  def test_cache_dir_escapes_windows_paths
-    uri = URI.parse("file:///C:/WINDOWS/Temp/gem_repo")
-    cache_dir = @sf.cache_dir(uri).gsub(@sf.dir, '')
-    assert cache_dir !~ /:/, "#{cache_dir} should not contain a :"
-  end
 end
 
