@@ -86,7 +86,7 @@ if defined? Zlib
     def test_adler
       z = Zlib::Deflate.new
       z << "foo"
-      s = z.finish
+      z.finish
       assert_equal(0x02820145, z.adler)
     end
 
@@ -95,7 +95,7 @@ if defined? Zlib
       assert_equal(false, z.finished?)
       z << "foo"
       assert_equal(false, z.finished?)
-      s = z.finish
+      z.finish
       assert_equal(true, z.finished?)
       z.close
       assert_raise(Zlib::Error) { z.finished? }
@@ -363,6 +363,34 @@ if defined? Zlib
         assert_equal(3, gz.tell)
       end
     end
+
+    def test_path
+      t = Tempfile.new("test_zlib_gzip_file")
+      t.close
+
+      gz = Zlib::GzipWriter.open(t.path)
+      gz.print("foo")
+      assert_equal(t.path, gz.path)
+      gz.close
+      assert_equal(t.path, gz.path)
+
+      f = Zlib::GzipReader.open(t.path)
+      assert_equal(t.path, f.path)
+      f.close
+      assert_equal(t.path, f.path)
+
+      s = ""
+      sio = StringIO.new(s)
+      gz = Zlib::GzipWriter.new(sio)
+      gz.print("foo")
+      assert_raise(NoMethodError) { gz.path }
+      gz.close
+
+      sio = StringIO.new(s)
+      f = Zlib::GzipReader.new(sio)
+      assert_raise(NoMethodError) { f.path }
+      f.close
+    end
   end
 
   class TestZlibGzipReader < Test::Unit::TestCase
@@ -431,21 +459,51 @@ if defined? Zlib
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
-      f = Zlib::GzipReader.open(t.path)
-      assert_equal("foo", f.read(3))
-      f.unused
-      assert_equal("bar", f.read)
-      f.unused
-      f.close
+      Zlib::GzipReader.open(t.path) do |f|
+        assert_equal(nil, f.unused)
+        assert_equal("foo", f.read(3))
+        assert_equal(nil, f.unused)
+        assert_equal("bar", f.read)
+        assert_equal(nil, f.unused)
+      end
+    end
+
+    def test_unused2
+      zio = StringIO.new
+
+      io = Zlib::GzipWriter.new zio
+      io.write 'aaaa'
+      io.finish
+
+      io = Zlib::GzipWriter.new zio
+      io.write 'bbbb'
+      io.finish
+
+      zio.rewind
+
+      io = Zlib::GzipReader.new zio
+      assert_equal('aaaa', io.read)
+      unused = io.unused
+      assert_equal(24, unused.bytesize)
+      io.finish
+
+      zio.pos -= unused.length
+
+      io = Zlib::GzipReader.new zio
+      assert_equal('bbbb', io.read)
+      assert_equal(nil, io.unused)
+      io.finish
     end
 
     def test_read
       t = Tempfile.new("test_zlib_gzip_reader")
       t.close
-      Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
+      str = "\u3042\u3044\u3046"
+      Zlib::GzipWriter.open(t.path) {|gz| gz.print(str) }
 
-      f = Zlib::GzipReader.open(t.path)
+      f = Zlib::GzipReader.open(t.path, encoding: "UTF-8")
       assert_raise(ArgumentError) { f.read(-1) }
+      assert_equal(str, f.read)
     end
 
     def test_readpartial
@@ -470,17 +528,27 @@ if defined? Zlib
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
       f = Zlib::GzipReader.open(t.path)
-      "foobar".each_byte {|c| assert_equal(c, f.getc) }
+      "foobar".each_char {|c| assert_equal(c, f.getc) }
       assert_nil(f.getc)
     end
 
+    def test_getbyte
+      t = Tempfile.new("test_zlib_gzip_reader")
+      t.close
+      Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
+
+      f = Zlib::GzipReader.open(t.path)
+      "foobar".each_byte {|c| assert_equal(c, f.getbyte) }
+      assert_nil(f.getbyte)
+    end
+
     def test_readchar
       t = Tempfile.new("test_zlib_gzip_reader")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foobar") }
 
       f = Zlib::GzipReader.open(t.path)
-      "foobar".each_byte {|c| assert_equal(c, f.readchar) }
+      "foobar".each_byte {|c| assert_equal(c, f.readchar.ord) }
       assert_raise(EOFError) { f.readchar }
     end
 
@@ -495,7 +563,7 @@ if defined? Zlib
       assert_equal("foobar".each_byte.to_a, a)
     end
 
-    def test_gets
+    def test_gets2
       t = Tempfile.new("test_zlib_gzip_reader")
       t.close
       Zlib::GzipWriter.open(t.path) {|gz| gz.print("foo\nbar\nbaz\n") }
@@ -622,6 +690,16 @@ if defined? Zlib
       assert_equal(0x8a62c964, Zlib.adler32("abc\x01\x02\x03" * 10000))
     end
 
+    def test_adler32_combine
+      one = Zlib.adler32("fo")
+      two = Zlib.adler32("o")
+      begin
+        assert_equal(0x02820145, Zlib.adler32_combine(one, two, 1))
+      rescue NotImplementedError
+        skip "adler32_combine is not implemented"
+      end
+    end
+
     def test_crc32
       assert_equal(0x00000000, Zlib.crc32)
       assert_equal(0x8c736521, Zlib.crc32("foo"))
@@ -629,6 +707,16 @@ if defined? Zlib
       assert_equal(0x07f0d68f, Zlib.crc32("abc\x01\x02\x03" * 10000))
     end
 
+    def test_crc32_combine
+      one = Zlib.crc32("fo")
+      two = Zlib.crc32("o")
+      begin
+        assert_equal(0x8c736521, Zlib.crc32_combine(one, two, 1))
+      rescue NotImplementedError
+        skip "crc32_combine is not implemented"
+      end
+    end
+
     def test_crc_table
       t = Zlib.crc_table
       assert_instance_of(Array, t)
